/*
* Copyright (c) Gengtu Technology Ltd. All Rights Reserved
* Dual MIT/GPLv2
*
* The contents of this file are subject to the MIT license as set out below.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Alternatively, the contents of this file may be used under the terms of
* the GNU General Public License Version 2 ("GPL") in which case the provisions
* of GPL are applicable instead of those above.
*
* If you wish to allow use of your version of this file only under the terms of
* GPL, and not to allow others to use your version of this file under the terms
* of the MIT license, indicate your decision by deleting the provisions above
* and replace them with the notice and other provisions required by GPL as set
* out in the file called "GPL-COPYING" included in this distribution. If you do
* not delete the provisions above, a recipient may use your version of this file
* under the terms of either the MIT license or GPL.
*
* This License is also included in this distribution in the file called
* "MIT-COPYING".
*
* EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
* PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
* BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#ifndef __GT_DRM_H__
#define __GT_DRM_H__

#include <linux/types.h>

typedef void gt_drm_file;
typedef void gt_drm_device;
typedef void gt_drm_scdc;
typedef void gt_drm_connector;
typedef void gt_edid;
typedef void gt_drm_crtc;
typedef void gt_drm_plane;
typedef void gt_drm_plane_state;
typedef void gt_drm_framebuffer;
typedef void gt_drm_format_info;
typedef void gt_drm_crtc_state;
typedef void gt_drm_connector_state;
typedef void gt_videomode;
typedef void gt_drm_mode_object;
typedef void gt_drm_gem_object;
typedef void gt_drm_dp_aux;
typedef void gt_drm_dp_link;

struct gt_drm_format_name_buf{
	char str[32];
};



void *gt_get_drm_file_prvdata(gt_drm_file *file);
void gt_set_drm_file_prvdata(gt_drm_file *file, void *data);

bool gt_is_support_scdc(gt_drm_scdc *scdc);
bool gt_connector_is_support_scdc(gt_drm_connector *connector);

ssize_t gt_drm_scdc_read(void *adapter, u8 offset, void *buffer,
		      size_t size);
ssize_t gt_drm_scdc_write(void *adapter, u8 offset,
		       const void *buffer, size_t size);
gt_edid *gt_drm_do_get_edid(gt_drm_connector *connector,
	int (*get_edid_block)(void *data, u8 *buf, unsigned int block,
			      size_t len), void *data);
int gt_drm_edid_header_is_valid(const u8 *raw_edid);
int gt_drm_edid_block_checksum(const u8 *raw_edid);
int gt_drm_do_probe_ddc_edid(void *data, u8 *buf, unsigned int block, size_t len);
bool gt_drm_irq_enabled(gt_drm_device *drm_dev);
bool gt_drm_crtc_handle_vblank(gt_drm_crtc *crtc);
void *gt_drm_dev_get_dev(gt_drm_device *drm_dev);
void *gt_drm_dev_get_prvdata(gt_drm_device *drm_dev);
bool gt_dp_drm_helper_hpd_irq_event(gt_drm_device *dev);


gt_drm_device *gt_drm_plane_get_drmdev_dev(gt_drm_plane *plane);
gt_drm_device *gt_drm_plane_get_drmdev(gt_drm_plane *plane);
char *gt_drm_plane_get_name(gt_drm_plane *plane);
void *gt_drm_plane_get_drmdev_private(gt_drm_plane *plane);
gt_drm_plane_state *gt_drm_plane_get_state(gt_drm_plane *plane);
#define gt_drm_plane_get_member(member, plane) gt_drm_plane_get_##member(plane)


unsigned int gt_drm_plane_state_get_rotation(gt_drm_plane_state *state);
unsigned int gt_drm_plane_state_get_src_w(gt_drm_plane_state *state);
unsigned int gt_drm_plane_state_get_src_h(gt_drm_plane_state *state);
unsigned int gt_drm_plane_state_get_src_x(gt_drm_plane_state *state);
unsigned int gt_drm_plane_state_get_src_y(gt_drm_plane_state *state);
unsigned int gt_drm_plane_state_get_crtc_w(gt_drm_plane_state *state);
unsigned int gt_drm_plane_state_get_crtc_h(gt_drm_plane_state *state);
int gt_drm_plane_state_get_crtc_x(gt_drm_plane_state *state);
int gt_drm_plane_state_get_crtc_y(gt_drm_plane_state *state);
void *gt_drm_plane_state_get_fb(gt_drm_plane_state *state);
void *gt_drm_plane_state_get_crtc(gt_drm_plane_state *state);
#define gt_drm_plane_state_get_member(member, state) gt_drm_plane_state_get_##member(state)



unsigned int *gt_drm_fb_get_pitches(gt_drm_framebuffer *fb);
unsigned int *gt_drm_fb_get_offsets(gt_drm_framebuffer *fb);
uint64_t gt_drm_fb_get_modifier(gt_drm_framebuffer *fb);
const gt_drm_format_info *gt_drm_fb_get_format_info(gt_drm_framebuffer *fb);
#define gt_drm_fb_get_member(member, fb) gt_drm_fb_get_##member(fb)



u32  gt_drm_framebuffer_get_format(gt_drm_framebuffer *fb);
void gt_drm_fb_get_bpp_depth(u32 format, u32 *depth, u32 *bpp);
u32  gt_ilog2(u32 n);

unsigned int gt_drm_format_info_get_format(const gt_drm_format_info *format);
unsigned char *gt_drm_format_info_get_cpp(const gt_drm_format_info *format);
unsigned char gt_drm_fb_get_cpp(gt_drm_framebuffer *fb, unsigned char index);

#define gt_drm_format_info_get_member(member, format) gt_drm_format_info_get_##member(format)



void *gt_drm_crtc_get_state(gt_drm_crtc *crtc);
void *gt_drm_crtc_get_name(gt_drm_crtc *crtc);
void *gt_drm_crtc_get_dev(gt_drm_crtc *crtc);
#define gt_drm_crtc_get_member(member, crtc) gt_drm_crtc_get_##member(crtc)


void *gt_drm_crtc_state_get_adjusted_mode(gt_drm_crtc_state *state);
void *gt_drm_crtc_state_get_ctm(gt_drm_crtc_state *state);
void *gt_drm_crtc_state_get_gamma_lut(gt_drm_crtc_state *state);
bool gt_drm_crtc_state_get_color_mgmt_changed(gt_drm_crtc_state *state);
bool gt_drm_crtc_state_get_enable(gt_drm_crtc_state *state);
unsigned int gt_drm_crtc_state_get_ctm_baseid(gt_drm_crtc_state *state);
unsigned int gt_drm_crtc_state_get_gamma_baseid(gt_drm_crtc_state *state);
bool gt_drm_crtc_state_get_active(gt_drm_crtc_state *state);
#define gt_drm_crtc_state_get_member(member, crtc) gt_drm_crtc_state_get_##member(crtc)



gt_videomode *gt_drm_videomode_alloc(void);
void gt_drm_videomode_free(gt_videomode *mode);
unsigned int gt_drm_videomode_get_hactive(gt_videomode *mode);
unsigned int gt_drm_videomode_get_htotal(gt_videomode *mode);
unsigned int gt_drm_videomode_get_hfront_porch(gt_videomode *mode);
unsigned int gt_drm_videomode_get_hback_porch(gt_videomode *mode);
unsigned int gt_drm_videomode_get_hsync_len(gt_videomode *mode);
int gt_drm_videomode_get_vactive(gt_videomode *mode);
unsigned int gt_drm_videomode_get_vfront_porch(gt_videomode *mode);
unsigned int gt_drm_videomode_get_vback_porch(gt_videomode *mode);
unsigned int gt_drm_videomode_get_vsync_len(gt_videomode *mode);
unsigned int gt_drm_videomode_get_flags(gt_videomode *mode);
unsigned long gt_drm_videomode_get_pixelclock(gt_videomode *mode);
#define gt_drm_videomode_get_member(member, mode) gt_drm_videomode_get_##member(mode)

void gt_drm_get_format_name(u32 format, struct gt_drm_format_name_buf format_name);


u32 GT_DRM_FORMAT_ARGB2101010_FUNC(void);
u32 GT_DRM_FORMAT_ABGR2101010_FUNC(void);
u32 GT_DRM_FORMAT_RGBA1010102_FUNC(void);
u32 GT_DRM_FORMAT_BGRA1010102_FUNC(void);
u32 GT_DRM_FORMAT_ARGB8888_FUNC(void);
u32 GT_DRM_FORMAT_ABGR8888_FUNC(void);
u32 GT_DRM_FORMAT_RGBA8888_FUNC(void);
u32 GT_DRM_FORMAT_BGRA8888_FUNC(void);
u32 GT_DRM_FORMAT_XRGB8888_FUNC(void);
u32 GT_DRM_FORMAT_XBGR8888_FUNC(void);
u32 GT_DRM_FORMAT_RGBX8888_FUNC(void);
u32 GT_DRM_FORMAT_BGRX8888_FUNC(void);
u32 GT_DRM_FORMAT_RGB888_FUNC(void);
u32 GT_DRM_FORMAT_BGR888_FUNC(void);
u32 GT_DRM_FORMAT_RGBA5551_FUNC(void);
u32 GT_DRM_FORMAT_ABGR1555_FUNC(void);
u32 GT_DRM_FORMAT_RGB565_FUNC(void);
u32 GT_DRM_FORMAT_BGR565_FUNC(void);
u32 GT_DRM_FORMAT_UYVY_FUNC(void);
u32 GT_DRM_FORMAT_YUYV_FUNC(void);
u32 GT_DRM_FORMAT_NV12_FUNC(void);
u32 GT_DRM_FORMAT_YUV420_FUNC(void);



#define GT_DRM_FORMAT_ARGB2101010    GT_DRM_FORMAT_ARGB2101010_FUNC()
#define GT_DRM_FORMAT_ABGR2101010    GT_DRM_FORMAT_ABGR2101010_FUNC()
#define GT_DRM_FORMAT_RGBA1010102    GT_DRM_FORMAT_RGBA1010102_FUNC()
#define GT_DRM_FORMAT_BGRA1010102     GT_DRM_FORMAT_BGRA1010102_FUNC()
#define GT_DRM_FORMAT_ARGB8888        GT_DRM_FORMAT_ARGB8888_FUNC()
#define GT_DRM_FORMAT_ABGR8888        GT_DRM_FORMAT_ABGR8888_FUNC()
#define GT_DRM_FORMAT_RGBA8888       GT_DRM_FORMAT_RGBA8888_FUNC()
#define GT_DRM_FORMAT_BGRA8888       GT_DRM_FORMAT_BGRA8888_FUNC()
#define GT_DRM_FORMAT_XRGB8888        GT_DRM_FORMAT_XRGB8888_FUNC()
#define GT_DRM_FORMAT_XBGR8888        GT_DRM_FORMAT_XBGR8888_FUNC()
#define GT_DRM_FORMAT_RGBX8888        GT_DRM_FORMAT_RGBX8888_FUNC()
#define GT_DRM_FORMAT_BGRX8888       GT_DRM_FORMAT_BGRX8888_FUNC()
#define GT_DRM_FORMAT_RGB888          GT_DRM_FORMAT_RGB888_FUNC()
#define GT_DRM_FORMAT_BGR888         GT_DRM_FORMAT_BGR888_FUNC()
#define GT_DRM_FORMAT_RGBA5551        GT_DRM_FORMAT_RGBA5551_FUNC()
#define GT_DRM_FORMAT_ABGR1555        GT_DRM_FORMAT_ABGR1555_FUNC()
#define GT_DRM_FORMAT_RGB565         GT_DRM_FORMAT_RGB565_FUNC()
#define GT_DRM_FORMAT_BGR565          GT_DRM_FORMAT_BGR565_FUNC()
#define GT_DRM_FORMAT_UYVY         GT_DRM_FORMAT_UYVY_FUNC()
#define GT_DRM_FORMAT_YUYV          GT_DRM_FORMAT_YUYV_FUNC()
#define GT_DRM_FORMAT_NV12         GT_DRM_FORMAT_NV12_FUNC()
#define GT_DRM_FORMAT_YUV420          GT_DRM_FORMAT_YUV420_FUNC()



u32 GT_DISPLAY_FLAGS_HSYNC_HIGH_FUNC(void);
u32 GT_DISPLAY_FLAGS_VSYNC_HIGH_FUNC(void);
u32 GT_DISPLAY_FLAGS_INTERLACED_FUNC(void);


#define GT_DISPLAY_FLAGS_HSYNC_HIGH   GT_DISPLAY_FLAGS_HSYNC_HIGH_FUNC()
#define GT_DISPLAY_FLAGS_VSYNC_HIGH   GT_DISPLAY_FLAGS_VSYNC_HIGH_FUNC()
#define GT_DISPLAY_FLAGS_INTERLACED   GT_DISPLAY_FLAGS_INTERLACED_FUNC()

u32 connector_status_connected_func(void);
u32 connector_status_disconnected_func(void);
u32 connector_status_unknown_func(void);
#define gt_connector_status_unknown		connector_status_unknown_func()
#define gt_connector_status_connected		connector_status_connected_func()
#define gt_connector_status_disconnected	connector_status_disconnected_func()
const char *gt_drm_get_connector_status_name(int);

u32 GT_DP_AUX_NATIVE_REPLY_ACK_FUNC(void);
u32 GT_DP_AUX_NATIVE_REPLY_NACK_FUNC(void);
u32 GT_DP_AUX_NATIVE_REPLY_DEFER_FUNC(void);
u32 GT_DP_AUX_NATIVE_REPLY_MASK_FUNC(void);
#define GT_DP_AUX_NATIVE_REPLY_ACK     GT_DP_AUX_NATIVE_REPLY_ACK_FUNC()
#define GT_DP_AUX_NATIVE_REPLY_NACK    GT_DP_AUX_NATIVE_REPLY_NACK_FUNC()
#define GT_DP_AUX_NATIVE_REPLY_DEFER   GT_DP_AUX_NATIVE_REPLY_DEFER_FUNC()
#define GT_DP_AUX_NATIVE_REPLY_MASK    GT_DP_AUX_NATIVE_REPLY_MASK_FUNC()


u32  GT_DRM_MODE_ROTATE_MASK_FUNC(void);
u32  GT_DRM_MODE_ROTATE_0_FUNC(void);
u32  GT_DRM_MODE_ROTATE_90_FUNC(void);
u32  GT_DRM_MODE_ROTATE_180_FUNC(void);
u32  GT_DRM_MODE_ROTATE_270_FUNC(void);
#define GT_DRM_MODE_ROTATE_MASK		GT_DRM_MODE_ROTATE_MASK_FUNC()
#define GT_DRM_MODE_ROTATE_0			GT_DRM_MODE_ROTATE_0_FUNC()
#define GT_DRM_MODE_ROTATE_90			GT_DRM_MODE_ROTATE_90_FUNC()
#define GT_DRM_MODE_ROTATE_180		GT_DRM_MODE_ROTATE_180_FUNC()
#define GT_DRM_MODE_ROTATE_270		GT_DRM_MODE_ROTATE_270_FUNC()

u32  GT_DRM_MODE_REFLECT_X_FUNC(void);
u32  GT_DRM_MODE_REFLECT_Y_FUNC(void);
#define GT_DRM_MODE_REFLECT_X			GT_DRM_MODE_REFLECT_X_FUNC()
#define GT_DRM_MODE_REFLECT_Y			GT_DRM_MODE_REFLECT_Y_FUNC()


gt_drm_gem_object *gt_drm_gem_object_lookup(gt_drm_file * filp, uint32_t handle);
void gt_drm_gem_object_put(void *gem_obj);


#endif
