/*
* Copyright (c) Gengtu Technology Ltd. All Rights Reserved
* Dual MIT/GPLv2
*
* The contents of this file are subject to the MIT license as set out below.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Alternatively, the contents of this file may be used under the terms of
* the GNU General Public License Version 2 ("GPL") in which case the provisions
* of GPL are applicable instead of those above.
*
* If you wish to allow use of your version of this file only under the terms of
* GPL, and not to allow others to use your version of this file under the terms
* of the MIT license, indicate your decision by deleting the provisions above
* and replace them with the notice and other provisions required by GPL as set
* out in the file called "GPL-COPYING" included in this distribution. If you do
* not delete the provisions above, a recipient may use your version of this file
* under the terms of either the MIT license or GPL.
*
* This License is also included in this distribution in the file called
* "MIT-COPYING".
*
* EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
* PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
* BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef __GT_DRM_MODE___H__
#define __GT_DRM_MODE___H__

//#include <linux/types.h>
//#include "gt_misc.h"

typedef void gt_drm_display_mode;
typedef unsigned int gt_drm_mode_status;
typedef unsigned int gt_hdmi_picture_aspect;

gt_drm_mode_status GT_MODE_NOCLOCK_FUNC(void);
gt_drm_mode_status GT_MODE_OK_FUNC(void);
gt_drm_mode_status GT_MODE_HSYNC_FUNC(void);
gt_drm_mode_status GT_MODE_VSYNC_FUNC(void);
gt_drm_mode_status GT_MODE_BAD_FUNC(void);
gt_hdmi_picture_aspect GT_HDMI_PICTURE_ASPECT_16_9_FUNC(void);
unsigned int GT_DRM_MODE_FLAG_PHSYNC_FUNC(void);
unsigned int GT_DRM_MODE_FLAG_PVSYNC_FUNC(void);
unsigned int  GT_DRM_MODE_FLAG_INTERLACE_FUNC(void);


#define GT_MODE_NOCLOCK                       GT_MODE_NOCLOCK_FUNC()
#define GT_MODE_OK                            GT_MODE_OK_FUNC()
#define GT_MODE_HSYNC                         GT_MODE_HSYNC_FUNC()
#define GT_MODE_VSYNC                         GT_MODE_VSYNC_FUNC()
#define GT_MODE_BAD                           GT_MODE_BAD_FUNC()
#define GT_HDMI_PICTURE_ASPECT_16_9           GT_HDMI_PICTURE_ASPECT_16_9_FUNC()
#define GT_DRM_MODE_FLAG_PHSYNC               GT_DRM_MODE_FLAG_PHSYNC_FUNC()
#define GT_DRM_MODE_FLAG_PVSYNC               GT_DRM_MODE_FLAG_PVSYNC_FUNC()
#define GT_DRM_MODE_FLAG_INTERLACE            GT_DRM_MODE_FLAG_INTERLACE_FUNC()




int gt_drm_mode_vrefresh(const gt_drm_display_mode *mode);
unsigned char gt_drm_match_cea_mode(const gt_drm_display_mode *to_match);
bool gt_drm_helper_hpd_irq_event(gt_drm_device *dev);

unsigned int gt_drm_disp_get_htotal(const gt_drm_display_mode *mode);
unsigned int gt_drm_disp_get_hdisplay(const gt_drm_display_mode *mode);
unsigned int gt_drm_disp_get_vtotal(const gt_drm_display_mode *mode);
unsigned int gt_drm_disp_get_vdisplay(const gt_drm_display_mode *mode);
unsigned int gt_drm_disp_get_hdisplay(const gt_drm_display_mode *mode);
unsigned int gt_drm_disp_get_hsync_start(const gt_drm_display_mode *mode);
unsigned int gt_drm_disp_get_hsync_end(const gt_drm_display_mode *mode);
unsigned int gt_drm_disp_get_vsync_start(const gt_drm_display_mode *mode);
unsigned int gt_drm_disp_get_vsync_end(const gt_drm_display_mode *mode);
unsigned int gt_drm_disp_get_clock(const gt_drm_display_mode *mode);
unsigned int gt_drm_disp_get_flags(const gt_drm_display_mode *mode);


unsigned int gt_drm_disp_set_htotal(gt_drm_display_mode *mode, unsigned int val) ;
unsigned int gt_drm_disp_set_hdisplay(gt_drm_display_mode *mode, unsigned int val) ;
unsigned int gt_drm_disp_set_vtotal(gt_drm_display_mode *mode, unsigned int val) ;
unsigned int gt_drm_disp_set_vdisplay(gt_drm_display_mode *mode, unsigned int val) ;
unsigned int gt_drm_disp_set_hdisplay(gt_drm_display_mode *mode, unsigned int val) ;
unsigned int gt_drm_disp_set_hsync_start(gt_drm_display_mode *mode, unsigned int val) ;
unsigned int gt_drm_disp_set_hsync_end(gt_drm_display_mode *mode, unsigned int val) ;
unsigned int gt_drm_disp_set_vsync_start(gt_drm_display_mode *mode, unsigned int val) ;
unsigned int gt_drm_disp_set_vsync_end(gt_drm_display_mode *mode, unsigned int val) ;
unsigned int gt_drm_disp_set_clock(gt_drm_display_mode *mode, unsigned int val) ;
unsigned int gt_drm_disp_set_flags(gt_drm_display_mode *mode, unsigned int val) ;

gt_drm_display_mode *gt_drm_disp_mode_alloc(char *name,
                                    int clk, int hdisplay, int hsync_start,
                                    int hsync_end, int htotal, int hskew,
                                    int vdisplay, int vsync_start, int vsync_end,
                                    int vtotal, int vscan);

void gt_drm_disp_mode_free(gt_drm_display_mode *mode);

#define gt_drm_disp_get_member(member, mode) gt_drm_disp_get_##member(mode)
#define gt_drm_disp_set_member(member, val, mode) gt_drm_disp_set_##member(mode, val)

#endif
