/*
* Copyright (c) Gengtu Technology Ltd. All Rights Reserved
* Dual MIT/GPLv2
*
* The contents of this file are subject to the MIT license as set out below.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Alternatively, the contents of this file may be used under the terms of
* the GNU General Public License Version 2 ("GPL") in which case the provisions
* of GPL are applicable instead of those above.
*
* If you wish to allow use of your version of this file only under the terms of
* GPL, and not to allow others to use your version of this file under the terms
* of the MIT license, indicate your decision by deleting the provisions above
* and replace them with the notice and other provisions required by GPL as set
* out in the file called "GPL-COPYING" included in this distribution. If you do
* not delete the provisions above, a recipient may use your version of this file
* under the terms of either the MIT license or GPL.
*
* This License is also included in this distribution in the file called
* "MIT-COPYING".
*
* EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
* PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
* BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include <linux/slab.h>
#include <linux/mutex.h>
#include <linux/completion.h>
#include <linux/spinlock.h>
#include <linux/atomic.h>
#include <linux/device.h>
#include <linux/semaphore.h>
#include <linux/preempt.h>

#include "gt_misc.h"
#include "gt_lock.h"

unsigned long gt_mutex_size = sizeof(struct mutex);
unsigned long gt_rwsem_size = sizeof(struct rw_semaphore);
unsigned long gt_spinlock_size = sizeof(spinlock_t);
GT_EXT_SYM(gt_rwsem_size);
GT_EXT_SYM(gt_spinlock_size);

GT_EXT_SYM(gt_mutex_size);

void gt_preempt_disable(void)
{
	preempt_disable();
}
GT_EXT_SYM(gt_preempt_disable);

void gt_preempt_enable(void)
{
	preempt_enable();
}
GT_EXT_SYM(gt_preempt_enable);

void gt_up(gt_semaphore *sem)
{
	up((struct semaphore *)sem);
}
GT_EXT_SYM(gt_up);

void gt_down(gt_semaphore *sem)
{
	down((struct semaphore *)sem);
}
GT_EXT_SYM(gt_down);

int gt_down_interruptible(gt_semaphore *sem)
{
	return down_interruptible((struct semaphore *)sem);
}
GT_EXT_SYM(gt_down_interruptible);

void gt_init_rwsem(gt_rw_semaphore *rwsem)
{
	init_rwsem((struct rw_semaphore *)rwsem);
}
GT_EXT_SYM(gt_init_rwsem);

void gt_down_read(gt_rw_semaphore *rwsem)
{
	down_read((struct rw_semaphore *)rwsem);
}
GT_EXT_SYM(gt_down_read);

void gt_down_read_nested(gt_rw_semaphore *rwsem, unsigned int subclass)
{
	down_read_nested((struct rw_semaphore *)rwsem, subclass);
}
GT_EXT_SYM(gt_down_read_nested);

void gt_up_read(gt_rw_semaphore *rwsem)
{
	up_read((struct rw_semaphore *)rwsem);
}
GT_EXT_SYM(gt_up_read);

void gt_down_write(gt_rw_semaphore *rwsem)
{
	down_write((struct rw_semaphore *)rwsem);
}
GT_EXT_SYM(gt_down_write);

void gt_up_write(gt_rw_semaphore *rwsem)
{
	up_write((struct rw_semaphore *)rwsem);
}
GT_EXT_SYM(gt_up_write);

void gt_mutex_init(gt_mutex *mutex)
{
	mutex_init((struct mutex *)mutex);
}
GT_EXT_SYM(gt_mutex_init);

void gt_mutex_destroy(gt_mutex *mutex)
{
	mutex_destroy((struct mutex *)mutex);
}
GT_EXT_SYM(gt_mutex_destroy);

gt_mutex *gt_mutex_alloc(void)
{
	struct mutex *m = (struct mutex *)kmalloc(sizeof(struct mutex), GFP_KERNEL);
	if (!m)
		return NULL;
	mutex_init(m);
	return m;
}
GT_EXT_SYM(gt_mutex_alloc);

void gt_mutex_free(gt_mutex *mutex)
{
	mutex_destroy((struct mutex*)mutex);

	kfree(mutex);
}
GT_EXT_SYM(gt_mutex_free);

void gt_mutex_lock(gt_mutex *mutex)
{
	mutex_lock((struct mutex *)mutex);
}
GT_EXT_SYM(gt_mutex_lock);

int gt_mutex_lock_interruptible(gt_mutex *mutex)
{
	return mutex_lock_interruptible((struct mutex *)mutex);
}
GT_EXT_SYM(gt_mutex_lock_interruptible);

void gt_mutex_lock_nested(gt_mutex *mutex, unsigned int subclass)
{
	mutex_lock_nested((struct mutex *)mutex, subclass);
}
GT_EXT_SYM(gt_mutex_lock_nested);

int gt_mutex_trylock(gt_mutex *mutex)
{
	return mutex_trylock((struct mutex *)mutex);
}
GT_EXT_SYM(gt_mutex_trylock);

void gt_mutex_unlock(gt_mutex *mutex)
{
	mutex_unlock((struct mutex *)mutex);
}
GT_EXT_SYM(gt_mutex_unlock);

int gt_mutex_is_locked(gt_mutex *mutex)
{
	return mutex_is_locked((struct mutex *)mutex);
}
GT_EXT_SYM(gt_mutex_is_locked);

gt_completion *gt_completion_alloc(void)
{
	struct completion *c = (struct completion *)kmalloc(sizeof(struct completion), GFP_KERNEL);
	if (!c)
		return NULL;
	init_completion(c);
	return c;
}
GT_EXT_SYM(gt_completion_alloc);

void gt_completion_free(gt_completion *c)
{
	kfree(c);
}
GT_EXT_SYM(gt_completion_free);

void gt_wait_for_completion(gt_completion *c)
{
	wait_for_completion((struct completion *)c);
}
GT_EXT_SYM(gt_wait_for_completion);

unsigned long gt_wait_for_completion_timeout(gt_completion *c, unsigned long timeout)
{
	return wait_for_completion_timeout((struct completion *)c, timeout);
}
GT_EXT_SYM(gt_wait_for_completion_timeout);

void gt_complete(gt_completion *c)
{
	complete((struct completion *)c);
}
GT_EXT_SYM(gt_complete);

void gt_init_completion(gt_completion *c)
{
	init_completion((struct completion *) c);
}
GT_EXT_SYM(gt_init_completion);

void gt_reinit_completion(gt_completion *c)
{
	reinit_completion((struct completion *) c);
}
GT_EXT_SYM(gt_reinit_completion);

void gt_spinlock_init(gt_spinlock *lock)
{
	spin_lock_init((spinlock_t *)lock);
}
GT_EXT_SYM(gt_spinlock_init);

gt_spinlock *gt_spinlock_alloc(void)
{
	spinlock_t *lock = (spinlock_t *)kmalloc(sizeof(spinlock_t), GFP_KERNEL);
	if (!lock)
		return NULL;
	spin_lock_init(lock);
	return lock;
}
GT_EXT_SYM(gt_spinlock_alloc);

gt_spinlock *gt_spinlock_devm_alloc(gt_dev *dev)
{
	spinlock_t *lock = devm_kmalloc((struct device *)dev, sizeof(spinlock_t),
			GFP_KERNEL);
	if (!lock)
		return NULL;
	spin_lock_init(lock);
	return lock;
}

void gt_spinlock_devm_free(gt_dev *dev, gt_spinlock *lock)
{
	devm_kfree((struct device *)dev, lock);
}

void gt_spinlock_free(gt_spinlock *lock)
{
	kfree(lock);
}
GT_EXT_SYM(gt_spinlock_free);

void gt_spin_lock_irqsave(gt_spinlock *lock, unsigned long *flags)
{
	spin_lock_irqsave((spinlock_t *)lock, (*flags));
}
GT_EXT_SYM(gt_spin_lock_irqsave);

void gt_spin_unlock_irqrestore(gt_spinlock *lock, unsigned long flags)
{
	spin_unlock_irqrestore((spinlock_t *)lock, flags);
}
GT_EXT_SYM(gt_spin_unlock_irqrestore);

void gt_spin_lock(gt_spinlock *lock)
{
	spin_lock((spinlock_t *)lock);
}
GT_EXT_SYM(gt_spin_lock);

void gt_spin_unlock(gt_spinlock *lock)
{
	spin_unlock((spinlock_t *)lock);
}
GT_EXT_SYM(gt_spin_unlock);

void gt_spin_lock_irq(gt_spinlock *lock)
{
	spin_lock_irq((spinlock_t *)lock);
}
GT_EXT_SYM(gt_spin_lock_irq);

void gt_spin_unlock_irq(gt_spinlock *lock)
{
	spin_unlock_irq((spinlock_t *)lock);
}
GT_EXT_SYM(gt_spin_unlock_irq);

int32_t gt_atomic_read(const atomic_t *counter)
{
	return atomic_read(counter);
}
GT_EXT_SYM(gt_atomic_read);

void gt_atomic_write(atomic_t *counter, int32_t v)
{
	atomic_set(counter, v);
}
GT_EXT_SYM(gt_atomic_write);

void gt_atomic_inc(atomic_t *counter)
{
	atomic_inc(counter);
}
GT_EXT_SYM(gt_atomic_inc);

void gt_atomic_dec(atomic_t *counter)
{
	atomic_dec(counter);
}
GT_EXT_SYM(gt_atomic_dec);

int32_t gt_atomic_inc_return(atomic_t *counter)
{
	return atomic_inc_return(counter);
}
GT_EXT_SYM(gt_atomic_inc_return);

int32_t gt_atomic_dec_return(atomic_t *counter)
{
	return atomic_dec_return(counter);
}
GT_EXT_SYM(gt_atomic_dec_return);

int32_t gt_atomic_cmpxchg(atomic_t *counter, int old, int new)
{
	return atomic_cmpxchg(counter, old, new);
}
GT_EXT_SYM(gt_atomic_cmpxchg);

int32_t gt_atomic_xchg(atomic_t *counter, int v)
{
	return atomic_xchg(counter, v);
}
GT_EXT_SYM(gt_atomic_xchg);

int32_t gt_atomic_add_return(int32_t v, atomic_t *counter)
{
	return atomic_add_return(v, counter);
}
GT_EXT_SYM(gt_atomic_add_return);

bool gt_atomic_add_unless(atomic_t *counter, int a, int u)
{
	return atomic_add_unless(counter, a, u);
}
GT_EXT_SYM(gt_atomic_add_unless);

int32_t gt_atomic_sub_return(atomic_t *counter, int32_t v)
{
	return atomic_sub_return(v, counter);
}
GT_EXT_SYM(gt_atomic_sub_return);

gt_rwlock* gt_rwlock_alloc(void)
{
	rwlock_t *rw_lock = NULL;

	rw_lock = kmalloc(sizeof(rwlock_t), GFP_KERNEL);
	if (!rw_lock)
		return rw_lock;
	else
		rwlock_init(rw_lock);

	return (gt_rwlock *)rw_lock;
}
GT_EXT_SYM(gt_rwlock_alloc);

void gt_rwlock_free(gt_rwlock *rw_lock)
{
	if (!rw_lock)
		return;
	
	kfree(rw_lock);
}
GT_EXT_SYM(gt_rwlock_free);

void gt_read_lock(gt_rwlock *rw_lock)
{
	read_lock((rwlock_t *)rw_lock);
}
GT_EXT_SYM(gt_read_lock);

void gt_read_unlock(gt_rwlock *rw_lock)
{
	read_unlock((rwlock_t *)rw_lock);
}
GT_EXT_SYM(gt_read_unlock);

void gt_write_lock(gt_rwlock *rw_lock)
{
	write_lock((rwlock_t *)rw_lock);
}
GT_EXT_SYM(gt_write_lock);

void gt_write_unlock(gt_rwlock *rw_lock)
{
	write_unlock((rwlock_t *)rw_lock);
}
GT_EXT_SYM(gt_write_unlock);
