/*
* Copyright (c) Gengtu Technology Ltd. All Rights Reserved
* Dual MIT/GPLv2
*
* The contents of this file are subject to the MIT license as set out below.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Alternatively, the contents of this file may be used under the terms of
* the GNU General Public License Version 2 ("GPL") in which case the provisions
* of GPL are applicable instead of those above.
*
* If you wish to allow use of your version of this file only under the terms of
* GPL, and not to allow others to use your version of this file under the terms
* of the MIT license, indicate your decision by deleting the provisions above
* and replace them with the notice and other provisions required by GPL as set
* out in the file called "GPL-COPYING" included in this distribution. If you do
* not delete the provisions above, a recipient may use your version of this file
* under the terms of either the MIT license or GPL.
*
* This License is also included in this distribution in the file called
* "MIT-COPYING".
*
* EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
* PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
* BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef __GT_LOCK_H__
#define __GT_LOCK_H__

#include <linux/types.h>
#include "gt_plat_dev.h"

typedef void gt_mutex;
typedef void gt_completion;
typedef void gt_spinlock;
typedef void gt_rw_semaphore;
typedef void gt_semaphore;
typedef void gt_rwlock;

extern unsigned long gt_rwsem_size;
extern unsigned long gt_mutex_size;
extern unsigned long gt_spinlock_size;

void gt_preempt_disable(void);

void gt_preempt_enable(void);

void gt_up(gt_semaphore *sem);

void gt_down(gt_semaphore *sem);

int gt_down_interruptible(gt_semaphore *sem);

void gt_init_rwsem(gt_rw_semaphore *rwsem);

void gt_down_read(gt_rw_semaphore *rwsem);

void gt_down_read_nested(gt_rw_semaphore *rwsem, unsigned int subclass);

void gt_up_read(gt_rw_semaphore *rwsem);

void gt_down_write(gt_rw_semaphore *rwsem);

void gt_up_write(gt_rw_semaphore *rwsem);

void gt_mutex_init(gt_mutex *mutex);

void gt_mutex_destroy(gt_mutex *mutex);

gt_mutex *gt_mutex_alloc(void);

void gt_mutex_free(gt_mutex *mutex);

void gt_mutex_lock(gt_mutex *mutex);

int gt_mutex_lock_interruptible(gt_mutex *mutex);

void gt_mutex_lock_nested(gt_mutex *mutex, unsigned int subclass);

void gt_mutex_unlock(gt_mutex *mutex);

int gt_mutex_trylock(gt_mutex *mutex);

int gt_mutex_is_locked(gt_mutex *mutex);

gt_completion *gt_completion_alloc(void);

void gt_completion_free(gt_completion *);

void gt_wait_for_completion(gt_completion *);

unsigned long gt_wait_for_completion_timeout(gt_completion *c, unsigned long timeout);

void gt_complete(gt_completion *);

void gt_init_completion(gt_completion *);

void gt_reinit_completion(gt_completion *);

void gt_spinlock_init(gt_spinlock *lock);

gt_spinlock *gt_spinlock_alloc(void);

void gt_spinlock_free(gt_spinlock *);

gt_spinlock *gt_spinlock_devm_alloc(gt_dev *dev);

void gt_spinlock_devm_free(gt_dev *dev, gt_spinlock *lock);

void gt_spin_lock_irqsave(gt_spinlock *, unsigned long *flags);

void gt_spin_unlock_irqrestore(gt_spinlock *, unsigned long flags);

void gt_spin_lock(gt_spinlock *lock);

void gt_spin_unlock(gt_spinlock *lock);

void gt_spin_lock_irq(gt_spinlock *lock);

void gt_spin_unlock_irq(gt_spinlock *lock);

int32_t gt_atomic_read(const atomic_t *counter);

void gt_atomic_write(atomic_t *counter, int32_t v);

void gt_atomic_inc(atomic_t *counter);

void gt_atomic_dec(atomic_t *counter);

int32_t gt_atomic_inc_return(atomic_t *counter);

int32_t gt_atomic_dec_return(atomic_t *counter);

int32_t gt_atomic_cmpxchg(atomic_t *counter, int old, int new);

int32_t gt_atomic_xchg(atomic_t *counter, int v);

int32_t gt_atomic_add_return(int32_t v, atomic_t *counter);

bool gt_atomic_add_unless(atomic_t *counter, int a, int u);

int32_t gt_atomic_sub_return(atomic_t *counter, int32_t v);

gt_rwlock* gt_rwlock_alloc(void);

void gt_rwlock_free(gt_rwlock *rw_lock);

void gt_read_lock(gt_rwlock *rw_lock);

void gt_read_unlock(gt_rwlock *rw_lock);

void gt_write_lock(gt_rwlock *rw_lock);

void gt_write_unlock(gt_rwlock *rw_lock);
#endif
