/*
 * Copyright (c) Gengtu Technology Ltd. All Rights Reserved
 * Dual MIT/GPLv2
 *
 * The contents of this file are subject to the MIT license as set out below.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 ("GPL") in which case the provisions
 * of GPL are applicable instead of those above.
 *
 * If you wish to allow use of your version of this file only under the terms of
 * GPL, and not to allow others to use your version of this file under the terms
 * of the MIT license, indicate your decision by deleting the provisions above
 * and replace them with the notice and other provisions required by GPL as set
 * out in the file called "GPL-COPYING" included in this distribution. If you do
 * not delete the provisions above, a recipient may use your version of this file
 * under the terms of either the MIT license or GPL.
 *
 * This License is also included in this distribution in the file called
 * "MIT-COPYING".
 *
 * EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
 * PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#include <generated/utsrelease.h>
#include <linux/pci.h>
#include <uapi/linux/pci.h>
#include <uapi/linux/pci_regs.h>
#include <linux/version.h>
#include "gt_misc.h"
#include "gt_pci.h"
#include "gt_debug.h"
#define KBUILD_GT_PCI "gt_pci"
#define pr_fmt_gt_pci(fmt) "[%s][%s:%d]" fmt,KBUILD_GT_PCI,__func__,__LINE__

#define gt_pci_notice(dev,fmt, ...)\
		gt_dev_printk(KERN_NOTICE,dev,pr_fmt_gt_pci(fmt), ##__VA_ARGS__)
#define gt_pci_error(dev,fmt, ...)\
		gt_dev_printk(KERN_ERR,dev,pr_fmt_gt_pci(fmt), ##__VA_ARGS__)

int GT_IRQ_MODE_LEGACY = PCI_IRQ_LEGACY;
int GT_IRQ_MODE_MSI = PCI_IRQ_MSI;

int gt_pci_enable_device(gt_pci_dev *dev)
{
	return pci_enable_device((struct pci_dev *)dev);
}
GT_EXT_SYM(gt_pci_enable_device);

void gt_pci_set_master(gt_pci_dev *dev)
{
	pci_set_master((struct pci_dev *)dev);
}
GT_EXT_SYM(gt_pci_set_master);

int gt_pci_enable_msi(gt_pci_dev *dev)
{
	return pci_enable_msi((struct pci_dev *)dev);
}
GT_EXT_SYM(gt_pci_enable_msi);

gt_pci_dev * gt_pci_get_device(unsigned int vendor, unsigned int device)
{
	struct pci_dev *dev;
	dev = pci_get_device(vendor, device, NULL);
	return dev;
}
GT_EXT_SYM(gt_pci_get_device);

unsigned long long gt_pci_resource_len(gt_pci_dev *dev, int bar)
{
	return pci_resource_len((struct pci_dev *)dev, bar);
}
GT_EXT_SYM(gt_pci_resource_len);

unsigned long long gt_pci_resource_start(gt_pci_dev *dev, int bar)
{
	return pci_resource_start((struct pci_dev *)dev, bar);
}
GT_EXT_SYM(gt_pci_resource_start);

unsigned long long gt_pci_resource_end(gt_pci_dev *dev, int bar)
{
	return pci_resource_end((struct pci_dev *)dev, bar);
}
GT_EXT_SYM(gt_pci_resource_end);

unsigned long gt_pci_resource_flags(gt_pci_dev *dev, int bar)
{
	return pci_resource_flags((struct pci_dev *)dev, bar);
}
GT_EXT_SYM(gt_pci_resource_flags);

int gt_pci_request_region(gt_pci_dev *dev, int bar, const char *res_name)
{
	return pci_request_region((struct pci_dev *)dev, bar, res_name);
}
GT_EXT_SYM(gt_pci_request_region);

void gt_pci_release_region(gt_pci_dev *dev, int bar)
{
	pci_release_region((struct pci_dev *)dev, bar);
}
GT_EXT_SYM(gt_pci_release_region);

void gt_pci_disable_msi(gt_pci_dev *dev)
{
	pci_disable_msi((struct pci_dev *)dev);
}
GT_EXT_SYM(gt_pci_disable_msi);

void gt_pci_clear_master(gt_pci_dev *dev)
{
	pci_clear_master((struct pci_dev *)dev);
}
GT_EXT_SYM(gt_pci_clear_master);

void gt_pci_restore_state(gt_pci_dev *dev)
{
	pci_restore_state((struct pci_dev *)dev);
}
GT_EXT_SYM(gt_pci_restore_state);

void gt_pci_disable_device(gt_pci_dev *dev)
{
	pci_disable_device((struct pci_dev *)dev);
}
GT_EXT_SYM(gt_pci_disable_device);

int gt_pci_save_state(gt_pci_dev *dev)
{
	return pci_save_state((struct pci_dev *)dev);
}
GT_EXT_SYM(gt_pci_save_state);

int gt_pci_set_power_state(gt_pci_dev *dev, enum gt_pmsg_state state)
{
	switch (state) {
	case GT_PMSG_ON:
		return pci_set_power_state((struct pci_dev *)dev, pci_choose_state((struct pci_dev *)dev, PMSG_ON));
	case GT_PMSG_SUSPEND:
		return pci_set_power_state((struct pci_dev *)dev, pci_choose_state((struct pci_dev *)dev, PMSG_SUSPEND));
	default:
		return GT_PMSG_NONE;
	}
}
GT_EXT_SYM(gt_pci_set_power_state);

bool gt_pci_is_busmaster(gt_pci_dev *dev)
{
	return ((struct pci_dev*)dev)->is_busmaster;
}
GT_EXT_SYM(gt_pci_is_busmaster);

unsigned int gt_get_pci_irq(gt_pci_dev *dev)
{
	return ((struct pci_dev *)dev)->irq;
}
GT_EXT_SYM(gt_get_pci_irq);

unsigned short gt_get_pci_vendor(gt_pci_dev *dev)
{
	return ((struct pci_dev *)dev)->vendor;
}
GT_EXT_SYM(gt_get_pci_vendor);

unsigned short gt_get_pci_device(gt_pci_dev *dev)
{
	return ((struct pci_dev *)dev)->device;
}
GT_EXT_SYM(gt_get_pci_device);

unsigned short gt_get_pci_subvendor(gt_pci_dev *dev)
{
	return ((struct pci_dev *)dev)->subsystem_vendor;
}
GT_EXT_SYM(gt_get_pci_subvendor);

unsigned short gt_get_pci_subdevice(gt_pci_dev *dev)
{
	return ((struct pci_dev *)dev)->subsystem_device;
}
GT_EXT_SYM(gt_get_pci_subdevice);

void gt_arch_phys_wc_del(int handle)
{
	arch_phys_wc_del(handle);
}
GT_EXT_SYM(gt_arch_phys_wc_del);

void gt_arch_io_free_memtype_wc(unsigned long long start, unsigned long long size)
{
	arch_io_free_memtype_wc(start, size);
}
GT_EXT_SYM(gt_arch_io_free_memtype_wc);

void * gt_request_region(unsigned long long start, unsigned long long n, const char * name)
{
	return request_region(start, n, name);
}
GT_EXT_SYM(gt_request_region);

void * gt_request_mem_region(unsigned long long start, unsigned long long n, const char *name)
{
	return request_mem_region(start, n, name);
}
GT_EXT_SYM(gt_request_mem_region);

void gt_release_region(unsigned long long start, unsigned long long n)
{
	release_region(start, n);
}
GT_EXT_SYM(gt_release_region);

void gt_release_mem_region(unsigned long long start, unsigned long long n)
{
	release_mem_region(start, n);
}
GT_EXT_SYM(gt_release_mem_region);

int gt_pci_irq_vector(gt_pci_dev *dev, unsigned int nr)
{
	return pci_irq_vector((struct pci_dev *)dev, nr);
}
GT_EXT_SYM(gt_pci_irq_vector);

gt_pci_dev *gt_to_pci_dev(void *dev)
{
	return (gt_pci_dev *)to_pci_dev((struct device *)dev);
}
GT_EXT_SYM(gt_to_pci_dev);

uint8_t gt_pci_revision_id(gt_pci_dev *dev)
{
	return ((struct pci_dev *)dev)->revision;
}
GT_EXT_SYM(gt_pci_revision_id);

int gt_pci_alloc_irq_vectors(gt_pci_dev *dev, unsigned int min_vecs, unsigned int max_vecs, int mode)
{
	return pci_alloc_irq_vectors((struct pci_dev *)dev, min_vecs, max_vecs, mode);
}
GT_EXT_SYM(gt_pci_alloc_irq_vectors);

void gt_pci_free_irq_vectors(gt_pci_dev *dev)
{
	return pci_free_irq_vectors((struct pci_dev *)dev);
}
GT_EXT_SYM(gt_pci_free_irq_vectors);

/*
 * When detect KylinOS2203-55 kernel and loongson 3a5000/7a2000 pcie bridge, return true (meaning pcie intx irq).
 * */
bool gt_loongarch_kylin2203_55_intx_irq(gt_pci_dev *dev)
{
#if defined(CONFIG_LOONGARCH) && defined(CONFIG_KYLINOS_DESKTOP)
#if (LINUX_VERSION_CODE == KERNEL_VERSION(5, 4, 18))
	/* Note: UTS_KYLINOS_RELEASE_ABI maybe: 55, 87.76, ky10, 77.66.sm2 etc for KylinOS release/debug version */
	if (strcmp(STRINGIFY(UTS_KYLINOS_RELEASE_ABI), "55") == 0) {
		struct pci_dev *pdev = (struct pci_dev *)dev;
		struct pci_bus *root = pdev->bus;

		while (root && root->self) {
			/*
			 * when root is pcie bridge, root->parent and root->self is non-null
			 * when root is pcie rc, root->parent and root->self is null
			 * 7a2000 pcie bridge vendorid: 0x14 deviceid: 0x7a59
			 * 7a1000 pcie bridge vendorid: 0x14 deviceid: 0x7a29
			 */
			if (root->self->vendor == 0x14 && root->self->device == 0x7a59) {
				return true;
			}

			root = root->parent;
		}
	}
#endif
#endif

	return false;
}
GT_EXT_SYM(gt_loongarch_kylin2203_55_intx_irq);


extern struct resource *pci_bus_resource_n(const struct pci_bus *bus, int n);
#define gt_pci_bus_for_each_resource(bus, res, i)				\
	for (i = 0;							\
		(res = pci_bus_resource_n(bus, i)) || i < PCI_BRIDGE_RESOURCE_NUM; \
		i++)

void gt_pci_release_resource(struct pci_dev *dev, int resno)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 15, 0))
	pci_release_resource(dev, resno);
#else
	// null
#endif

}

int gt_pci_resize_resource(struct pci_dev *dev, int resno, int size)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 15, 0))
	return pci_resize_resource(dev, resno, size);
#else
	return 1;
#endif
}

int gt_check_rc_size(gt_pci_dev *dev)
{
	struct pci_bus *root;
	struct resource *res;
	struct pci_dev *pdev;
	unsigned i;

	if (dev == NULL)
		return -1;

	pdev = (struct pci_dev *)dev;
	/* Check if the root BUS has 64bit memory resources */
	root = pdev->bus;
	while (root->parent)
		root = root->parent;

	gt_pci_bus_for_each_resource(root, res, i) {
		if (res && res->flags & (IORESOURCE_MEM | IORESOURCE_MEM_64) &&
			res->start > 0x100000000ull)
			break;
	}

	/* Trying to resize is pointless without a root hub window above 4GB */
	if (!res)
		return 1;

	return 0;
}

GT_EXT_SYM(gt_check_rc_size);

int gt_check_resize_version(void)
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 15, 0))
	return 0;
#else
	return 1;
#endif
}
GT_EXT_SYM(gt_check_resize_version);

#define GT_RESIZE_NUMS (0x1)
int gt_resize_bar(struct pci_dev *pdev, int resno, int rbar_size)
{
	u16 cmd;
	int ret = 0;
	u32 i;

	/* Disable memory decoding while we change the BAR addresses and size */
	pci_read_config_word(pdev, PCI_COMMAND, &cmd);
	pci_write_config_word(pdev, PCI_COMMAND,
				cmd & ~PCI_COMMAND_MEMORY);

	gt_pci_release_resource(pdev, resno);
	gt_pci_release_resource(pdev, 0);
	//gt_pci_error(&pdev->dev, " -ENOTSUPP:%d, -EBUSY:%d, -EINVAL:%d \n", (-ENOTSUPP), (-EBUSY), (-EINVAL) );
	for (i = 0; i < GT_RESIZE_NUMS; i++)
	{

		ret = gt_pci_resize_resource(pdev, resno, rbar_size);
		if(ret == -ENOSPC)
		{
			gt_pci_notice(&pdev->dev, "Not enough PCI address space for a large BAR.\n");
		}else if (ret && ret != -ENOTSUPP) {
				gt_pci_notice(&pdev->dev, "Problem resizing BAR2 (%d).retry:%d\n", ret, i);
		}else{
			gt_pci_notice(&pdev->dev, "pci_resize_resource cfg success \n");
			break;
		}

	}

	pci_assign_unassigned_bus_resources(pdev->bus);
	pci_write_config_word(pdev, PCI_COMMAND, cmd);

	return ret;
}

int gt_gpu_device_resize_fb_bar(gt_pci_dev *dev, int resno, int rbar_size, int *repo)
{
	struct pci_dev *pdev;
	int ret;

	if ((dev == NULL) || (repo == NULL))
		return -1;

	pdev = (struct pci_dev *)dev;
	ret = gt_resize_bar(pdev, resno, rbar_size);
	*repo = ret;

	return 0;
}

GT_EXT_SYM(gt_gpu_device_resize_fb_bar);

void* gt_device_under_switch(gt_pci_dev *dev)
{
	struct pci_dev *pdev;
	int dev_num;
	struct list_head *device;
	struct pci_bus *bus;

	if (dev == NULL)
		return NULL;

	pdev = (struct pci_dev *)dev;
	dev_num = 0;
	bus = pdev->bus;

	while (!pci_is_root_bus(bus))
	{
		list_for_each(device, &bus->devices)
		{
			++dev_num;
    	}
		if (dev_num > 1)
		{
			return bus->self;
		}
		else
		{
			bus = bus->parent;
		}
	}
	return NULL;
}

bool gt_devices_under_same_switch(gt_pci_dev *dev1, gt_pci_dev *dev2)
{
	gt_pci_dev *bridge1 = gt_device_under_switch(dev1);
	gt_pci_dev *bridge2 = gt_device_under_switch(dev2);
	if ((bridge1 != NULL) && (bridge2 != NULL) && (bridge1 == bridge2))
	{
		return true;
	}

	return false;
}
GT_EXT_SYM(gt_devices_under_same_switch);

bool gt_devices_under_same_RC(gt_pci_dev *pci_dev1, gt_pci_dev *pci_dev2)
{
	struct pci_bus *bus1;
	struct pci_bus *bus2;
	struct pci_dev *pdev1, *pdev2;
	if ((pci_dev1 == NULL) || (pci_dev2 == NULL))
		return NULL;

	pdev1 = (struct pci_dev *)pci_dev1;
	pdev2 = (struct pci_dev *)pci_dev2;

	bus1 = pdev1->bus;
	while (bus1->parent != NULL)
	{
		bus1 = bus1->parent;
	}

	bus2 = pdev2->bus;
	while (bus2->parent != NULL)
	{
		bus2 = bus2->parent;
	}

	if (bus1 == bus2)
	{
		return true;
	}
	return false;
}
GT_EXT_SYM(gt_devices_under_same_RC);
