/*
 * Copyright (c) Gengtu Technology Ltd. All Rights Reserved
 * Dual MIT/GPLv2
 *
 * The contents of this file are subject to the MIT license as set out below.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 ("GPL") in which case the provisions
 * of GPL are applicable instead of those above.
 *
 * If you wish to allow use of your version of this file only under the terms of
 * GPL, and not to allow others to use your version of this file under the terms
 * of the MIT license, indicate your decision by deleting the provisions above
 * and replace them with the notice and other provisions required by GPL as set
 * out in the file called "GPL-COPYING" included in this distribution. If you do
 * not delete the provisions above, a recipient may use your version of this file
 * under the terms of either the MIT license or GPL.
 *
 * This License is also included in this distribution in the file called
 * "MIT-COPYING".
 *
 * EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
 * PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#ifndef __GT_PCI_H__
#define __GT_PCI_H__

#include <linux/types.h>
enum gt_pmsg_state{
	GT_PMSG_ON,
	GT_PMSG_SUSPEND,
	GT_PMSG_NONE,
};

extern int GT_IRQ_MODE_LEGACY;
extern int GT_IRQ_MODE_MSI;

typedef void gt_pci_dev;

int gt_pci_enable_device(gt_pci_dev *dev);

void gt_pci_set_master(gt_pci_dev *dev);

int gt_pci_enable_msi(gt_pci_dev *dev);

gt_pci_dev * gt_pci_get_device(unsigned int vendor, unsigned int device);

int gt_check_resize_version(void);

int gt_check_rc_size(gt_pci_dev *dev);

int gt_gpu_device_resize_fb_bar(gt_pci_dev *dev, int resno, int rbar_size, int *repo);

unsigned long long gt_pci_resource_len(gt_pci_dev *dev, int bar);

unsigned long long gt_pci_resource_start(gt_pci_dev *dev, int bar);

unsigned long long gt_pci_resource_end(gt_pci_dev *dev, int bar);

unsigned long gt_pci_resource_flags(gt_pci_dev *dev, int bar);

int gt_pci_request_region(gt_pci_dev *dev, int bar, const char *res_name);

void gt_pci_release_region(gt_pci_dev *dev, int bar);

void gt_pci_disable_msi(gt_pci_dev *dev);

void gt_pci_clear_master(gt_pci_dev *dev);

void gt_pci_restore_state(gt_pci_dev *dev);

void gt_pci_disable_device(gt_pci_dev *dev);

int gt_pci_save_state(gt_pci_dev *dev);

int gt_pci_set_power_state(gt_pci_dev *dev, enum gt_pmsg_state state);

bool gt_pci_is_busmaster(gt_pci_dev *dev);

unsigned int gt_get_pci_irq(gt_pci_dev *dev);

unsigned short gt_get_pci_vendor(gt_pci_dev *dev);

unsigned short gt_get_pci_subvendor(gt_pci_dev *dev);

unsigned short gt_get_pci_device(gt_pci_dev *dev);

unsigned short gt_get_pci_subdevice(gt_pci_dev *dev);

int gt_pci_irq_vector(gt_pci_dev *dev, unsigned int nr);

void gt_arch_phys_wc_del(int handle);

void gt_arch_io_free_memtype_wc(unsigned long long start, unsigned long long size);

void * gt_request_region(unsigned long long start, unsigned long long n, const char * name);

void * gt_request_mem_region(unsigned long long start, unsigned long long n, const char *name);

void gt_release_region(unsigned long long start, unsigned long long n);

void gt_release_mem_region(unsigned long long start, unsigned long long n);

gt_pci_dev *gt_to_pci_dev(void *dev);

uint8_t gt_pci_revision_id(gt_pci_dev *dev);

bool gt_loongarch_kylin2203_55_intx_irq(gt_pci_dev *dev);

int gt_pci_alloc_irq_vectors(gt_pci_dev *dev, unsigned int min_vecs, unsigned int max_vecs, int mode);

void gt_pci_free_irq_vectors(gt_pci_dev *dev);

bool gt_devices_under_same_switch(gt_pci_dev *dev1, gt_pci_dev *dev2);

bool gt_devices_under_same_RC(gt_pci_dev *pci_dev1, gt_pci_dev *pci_dev2);
#endif
