/*
 * Copyright (c) Gengtu Technology Ltd. All Rights Reserved
 * Dual MIT/GPLv2
 *
 * The contents of this file are subject to the MIT license as set out below.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 ("GPL") in which case the provisions
 * of GPL are applicable instead of those above.
 *
 * If you wish to allow use of your version of this file only under the terms of
 * GPL, and not to allow others to use your version of this file under the terms
 * of the MIT license, indicate your decision by deleting the provisions above
 * and replace them with the notice and other provisions required by GPL as set
 * out in the file called "GPL-COPYING" included in this distribution. If you do
 * not delete the provisions above, a recipient may use your version of this file
 * under the terms of either the MIT license or GPL.
 *
 * This License is also included in this distribution in the file called
 * "MIT-COPYING".
 *
 * EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
 * PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#ifndef __GT_PLAT_DEV_H__
#define __GT_PLAT_DEV_H__

#include <linux/types.h>

typedef void gt_platform_device;
typedef void gt_resource;
typedef void gt_dev;
typedef void gt_misc_dev;
typedef void gt_drm;
typedef void gt_i2c_adapter;

struct gt_res {
	uint64_t start;
	uint64_t size;
	const char *name;
};

struct gt_plat_dev_info {
	gt_dev *parent;
	const char *name;
	int id;
	const void *data;
	size_t data_size;
	uint64_t dma_mask;
	struct gt_res *res;
	uint32_t num_res;
};

gt_platform_device *gt_platform_device_register_full(struct gt_plat_dev_info *);

void gt_platform_device_unregister(gt_platform_device *dev);

gt_dev *gt_platform_get_parent_dev(gt_platform_device *dev);

gt_dev *gt_platform_get_dev(gt_platform_device *dev);

gt_dev *gt_dev_get_parent(gt_dev *dev);

gt_platform_device *gt_to_platform_device(void *dev);

void *gt_platform_get_data(gt_platform_device *dev);

gt_resource *gt_platform_get_resource_byname(gt_platform_device *dev,
		unsigned int type, const char *name);

uint64_t gt_resource_start(gt_resource *res);

uint64_t gt_resource_size(gt_resource *res);

void *gt_rsrc_devres_alloc(size_t size);

void *gt_rsrc_devres_find(gt_dev *dev);

void gt_put_device(gt_dev *dev);

gt_dev *gt_get_device(gt_dev *dev);

void gt_platform_set_drvdata(void *pdev, void *data);
void *gt_platform_get_drvdata(void *pdev);

void *gt_dev_get_platdata(void *pdev);

void *gt_get_dev_ofnode(gt_dev *dev);

void *gt_get_platdev_drvdata(void *pdev);

void *gt_get_dev_drvdata(gt_dev *dev);

void gt_set_dev_drvdata(gt_dev *dev, void *data);

void *gt_get_drvdata(void *pdev);
int gt_platform_get_numa_node(gt_platform_device *dev);

int gt_dev_to_node(gt_dev *dev);

void gt_platform_suspend(gt_platform_device *dev);

uint64_t gt_platform_get_dma_mask(gt_platform_device *dev);

bool gt_get_iommu_enable(gt_dev* dev);

#endif

