#ifndef GT_POWER_H
#define GT_POWER_H

#include "hal.h"
#include <linux/version.h>

#define GT_POWER_ERR -1
#define GT_POWER_OK	0

#define POWER_CHIP_NAME	20

#define GTPWR "gtpwr"
#define pr_power(fmt) "[%s][%s:%d]" fmt,GTPWR,__func__,__LINE__

#define gtpwr_dbg(fmt, ...)\
	if (get_pwr_debug_lvl() >= PWRD_LVL_DBG) \
		printk(KERN_NOTICE pr_power(fmt), ##__VA_ARGS__)
#define gtpwr_info(fmt, ...)\
	if (get_pwr_debug_lvl() >= PWRD_LVL_INFO) \
		printk(KERN_NOTICE pr_power(fmt), ##__VA_ARGS__)
#define gtpwr_notice( fmt, ...) \
		printk(KERN_NOTICE pr_power(fmt), ##__VA_ARGS__)
#define gtpwr_warn( fmt, ...) \
		printk(KERN_WARNING pr_power(fmt), ##__VA_ARGS__)
#define gtpwr_err(fmt, ...)\
		printk(KERN_NOTICE pr_power(fmt), ##__VA_ARGS__)

struct powerchip {
	char name[POWER_CHIP_NAME];

	struct i2c_client *client;
	int (*get_gpu_voltage)(struct powerchip *pwrchip, int channel);
	int (*set_gpu_voltage)(struct powerchip *pwrchip, int channel, unsigned vol);

	struct mutex lock;
	void *priv_data;
};

struct gtpower {
	struct volctrl_chip_hwinfo_t *params;
	struct i2c_adapter *adapter;

	struct device *pdev;	//pci_dev->dev
	struct device *pltdev;

	struct powerchip *chip;
};

int gtpwr_read8(struct powerchip *pwrchip, unsigned char reg);
int gtpwr_write8(struct powerchip *pwrchip, unsigned char reg, unsigned char data);
int gtpwr_read16(struct powerchip *pwrchip, unsigned char reg);
int gtpwr_write16(struct powerchip *pwrchip, unsigned char reg, unsigned int data);

int gtpwr_read8_no_repeat_start(struct powerchip *pwrchip, unsigned char reg);
int gtpwr_write8_no_repeat_start(struct powerchip *pwrchip, unsigned char reg, unsigned char data);
int gtpwr_read16_no_repeat_start(struct powerchip *pwrchip, unsigned char reg);
int gtpwr_write16_no_repeat_start(struct powerchip *pwrchip, unsigned char reg, unsigned int data);

int gtpower_driver_register(void);
void gtpower_driver_unregister(void);
#include "power_hw_info.h"
#endif	/*GT_POWER_H*/
