
#include <linux/version.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 5, 0))
#include <drm/drm_drv.h>
#include <drm/drm_print.h>
#include <linux/mod_devicetable.h>
#include <linux/dma-mapping.h>
#include <linux/of.h>
#include <linux/slab.h>
#else
#include <drm/drmP.h>
#endif

#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/reboot.h>
#include <linux/notifier.h>

#include "module_common.h"
#include "gt69_drv.h"
#include "gt69module.h"
#include "sysinfo.h"
#include "gt69srv_error.h"
#include "gt69srv.h"

#include "gt69_sync.h"
#include "gt69_sync_api.h"

#include <linux/component.h>
#include "gtgpu_drm.h"


#if defined(SUPPORT_DISPLAY_CLASS)
/* Display class interface */
#include "kerneldisplay.h"
GT_EXT_SYM(DCRegisterDevice);
GT_EXT_SYM(DCUnregisterDevice);
GT_EXT_SYM(DCDisplayConfigurationRetired);
GT_EXT_SYM(DCDisplayHasPendingCommand);
GT_EXT_SYM(DCImportBufferAcquire);
GT_EXT_SYM(DCImportBufferRelease);

/* Physmem interface (required by LMA DC drivers) */
#include "physheap.h"
GT_EXT_SYM(PhysHeapAcquireByUsage);
GT_EXT_SYM(PhysHeapRelease);
GT_EXT_SYM(PhysHeapGetType);
GT_EXT_SYM(PhysHeapGetCpuPAddr);
GT_EXT_SYM(PhysHeapGetSize);
GT_EXT_SYM(PhysHeapCpuPAddrToDevPAddr);

GT_EXT_SYM(PVRSRVGetDriverStatus);
GT_EXT_SYM(PVRSRVSystemInstallDeviceLISR);
GT_EXT_SYM(PVRSRVSystemUninstallDeviceLISR);

#include "gt69_notifier.h"
GT_EXT_SYM(PVRSRVCheckStatus);

#include "gt69_debug.h"
GT_EXT_SYM(PVRSRVGetErrorString);
GT_EXT_SYM(PVRSRVGetDeviceInstance);
#endif /* defined(SUPPORT_DISPLAY_CLASS) */

#if defined(SUPPORT_RGX)
#include "gengtapi_km.h"
#if defined(SUPPORT_SHARED_SLC)
GT_EXT_SYM(RGXInitSLC);
#endif
GT_EXT_SYM(RGXHWPerfConnect);
GT_EXT_SYM(RGXHWPerfDisconnect);
GT_EXT_SYM(RGXHWPerfControl);
#if defined(HWPERF_PACKET_V2C_SIG)
GT_EXT_SYM(RGXHWPerfConfigureCounters);
#else
GT_EXT_SYM(RGXHWPerfConfigMuxCounters);
GT_EXT_SYM(RGXHWPerfConfigureAndEnableCustomCounters);
#endif
GT_EXT_SYM(RGXHWPerfDisableCounters);
GT_EXT_SYM(RGXHWPerfAcquireEvents);
GT_EXT_SYM(RGXHWPerfReleaseEvents);
GT_EXT_SYM(RGXHWPerfConvertCRTimeStamp);
#if defined(SUPPORT_KERNEL_HWPERF_TEST)
GT_EXT_SYM(OSAddTimer);
GT_EXT_SYM(OSEnableTimer);
GT_EXT_SYM(OSDisableTimer);
GT_EXT_SYM(OSRemoveTimer);
#endif
#endif

#include "gt_misc.h"
#include "gt_srvkm.h"
GT_EXT_SYM(gt_pmr_unref);
GT_EXT_SYM(gt_pmr_ref);
GT_EXT_SYM(gt_pmr_lock_sys_phys_addr);
GT_EXT_SYM(gt_pmr_unlock_sys_phys_addr);
GT_EXT_SYM(gt_stats_remove_mem_alloc_record);
GT_EXT_SYM(gt_stats_decr_mem_alloc_stat);
GT_EXT_SYM(gt_pmr_write_bytes);
GT_EXT_SYM(gt_pmr_read_bytes);


#if defined(__arm64__) || defined(__aarch64__) || defined(PVRSRV_DEVMEM_TEST_SAFE_MEMSETCPY)
#include "osfunc_common.h"

#if defined(__GNUC__)
/* Workarounds for assumptions made that memory will not be mapped uncached
 * in kernel or user address spaces on arm64 platforms (or other testing).
 * */

GT_EXT_SYM(DeviceMemSet);
GT_EXT_SYM(DeviceMemCopy);

#else /* defined __GNUC__ */

GT_EXT_SYM(DeviceMemSetBytes);
GT_EXT_SYM(DeviceMemCopyBytes);

#endif /* defined __GNUC__ */

#endif

#include "syscommon.h"
#include "osfunc.h"
GT_EXT_SYM(gpu_extern_vram_flag);
GT_EXT_SYM(OSMemoryBarrier);

/* This header must always be included last */
#include "kernel_compatibility.h"

/* Values used to configure the PVRSRV_DEVICE_INIT_MODE tunable (Linux-only) */
#define PVRSRV_LINUX_DEV_INIT_ON_PROBE   1
#define PVRSRV_LINUX_DEV_INIT_ON_OPEN    2
#define PVRSRV_LINUX_DEV_INIT_ON_CONNECT 3

static int gtgpu_gt69_bind(struct device *dev,
		struct device *master, void *data);
static void gtgpu_gt69_unbind(struct device *dev,
		struct device *master, void *data);

static DEFINE_MUTEX(g_device_mutex);

static int gt69_power_callback(struct notifier_block *gt69_power_notif, unsigned long event, void *ddev)
{
	struct gt69_drm_private *priv;

	priv = container_of(gt69_power_notif, struct gt69_drm_private, gt69_power_notifier);
	
	if (SYS_POWER_OFF == event)
	{
		priv->gt69_power_status = SYS_POWER_OFF;
	}
	else if (SYS_RESTART == event)
	{
		priv->gt69_power_status = SYS_RESTART;
	}

	return NOTIFY_DONE;	
}

extern struct platform_driver g_gtgpu_gt69srvkm_driver;
static const struct component_ops s_gtgpu_gt69_ops = {
	.bind = gtgpu_gt69_bind,
	.unbind = gtgpu_gt69_unbind,
};

static int gtgpu_gt69_drm_load(struct device *dev, unsigned long flags)
{
	struct platform_device *pdev = gt_to_platform_device(dev);
	plat_data_t *pdata = gt_platform_get_data(pdev);
	struct drm_device *drm_dev = NULL;
	struct gt69_drm_private *priv = NULL;
	enum PVRSRV_ERROR_TAG srv_err;
	int err, deviceId;

#if defined(NO_HARDWARE)
	drm_dev = gt_platform_get_drvdata(pdev);
	pdata->pdev_rsrc = NULL;
#else
	drm_dev = pdata->pdev_rsrc->ddev[pdata->dev_idx];
#endif
	if (drm_dev == NULL)
		DRM_ERROR("%s %d drm_dev is NULL\n", __func__, __LINE__);

	priv = gtgpu_drm_to_gt69_private(drm_dev);
	BUG_ON(!priv);
	DRM_DEBUG_DRIVER("device %p\n", drm_dev->dev);

	/*
	dev_set_drvdata(drm_dev->dev, drm_dev); // dropped, drm_dev->dev is pcie device, foribidden to use.
	*/

#if (LINUX_VERSION_CODE < KERNEL_VERSION(3, 12, 0))
	/*
	 * Older kernels do not have render drm_minor member in drm_device,
	 * so we fallback to primary node for device identification
	 */
	deviceId = drm_dev->primary->index;
#else
	if (drm_dev->render)
		deviceId = drm_dev->render->index;
	else /* when render node is NULL, fallback to primary node */
		deviceId = drm_dev->primary->index;
#endif

	/* dropped
	priv = kzalloc(sizeof(*priv), GFP_KERNEL);
	if (!priv) {
		err = -ENOMEM;
		goto err_exit;
	}
	drm_dev->dev_private = priv;

	// TBD ???
	if (!drm_dev->dev->dma_parms)
		drm_dev->dev->dma_parms = &priv->dma_parms;
	dma_set_max_seg_size(drm_dev->dev, DMA_BIT_MASK(32));
	*/

	mutex_lock(&g_device_mutex);

	srv_err = PVRSRVCommonDeviceCreate(dev, deviceId, &priv->dev_node);
	if (srv_err != PVRSRV_OK) {
		DRM_ERROR("failed to create device node for device %p (%s)\n",
			  drm_dev->dev, PVRSRVGetErrorString(srv_err));
		if (srv_err == PVRSRV_ERROR_PROBE_DEFER)
			err = -EPROBE_DEFER;
		else
			err = -ENODEV;
		goto err_unset_dma_parms;
	}

	srv_err = GTPVRSRVCommonDeviceCreate(priv->dev_node);
	if (srv_err != PVRSRV_OK) {
		DRM_ERROR("GTPVRSRVCommonDeviceCreate initialisation failed : %s\n", PVRSRVGetErrorString(srv_err));
		err = -EINVAL;
		goto err_device_destroy;
	}

	err = PVRSRVDeviceInit(priv->dev_node);
	if (err) {
		DRM_ERROR("device %p initialisation failed (err=%d)\n",
			  drm_dev->dev, err);
		goto err_gt_device_destroy;
	}
	/*
	drm_mode_config_init(drm_dev); // dropped
	*/

#if defined(SUPPORT_FWLOAD_ON_PROBE)
	srv_err = PVRSRVCommonDeviceInitialise(priv->dev_node);
	if (srv_err != PVRSRV_OK) {
		err = -ENODEV;
		DRM_ERROR("device %p initialisation failed (err=%d)\n",
			  drm_dev->dev, err);
		goto err_device_deinit;
	}
#endif

	mutex_unlock(&g_device_mutex);

	priv->gt69_power_status = 0;
	priv->gt69_power_notifier.notifier_call = gt69_power_callback;
	err = register_reboot_notifier(&priv->gt69_power_notifier);
	if (err) {
		DRM_ERROR("device %p gt69_power_notifier error \n", drm_dev->dev);
		goto err_reboot_register_fail;
	}

	return 0;

err_reboot_register_fail:
	mutex_lock(&g_device_mutex);
#if defined(SUPPORT_FWLOAD_ON_PROBE)
err_device_deinit:
	/*
	drm_mode_config_cleanup(drm_dev); // dropped
	*/
	PVRSRVDeviceDeinit(priv->dev_node);
#endif
err_gt_device_destroy:
	GTPVRSRVCommonDeviceDestroy(priv->dev_node);
err_device_destroy:
	PVRSRVCommonDeviceDestroy(priv->dev_node);
err_unset_dma_parms:
	mutex_unlock(&g_device_mutex);
	/* dropped
	if (drm_dev->dev->dma_parms == &priv->dma_parms)
		drm_dev->dev->dma_parms = NULL;
	kfree(priv);

err_exit:
	*/
	return err;
}


static int gtgpu_gt69_drm_unload(struct drm_device *drm_dev)
{
	struct gt69_drm_private *priv = gtgpu_drm_to_gt69_private(drm_dev);

	BUG_ON(!priv);

	DRM_DEBUG_DRIVER("device %p\n", drm_dev->dev);

	unregister_reboot_notifier(&priv->gt69_power_notifier);

	/*
	drm_mode_config_cleanup(drm_dev); // dropped
	*/

	PVRSRVDeviceDeinit(priv->dev_node);

	mutex_lock(&g_device_mutex); // TBD???
	GTPVRSRVCommonDeviceDestroy(priv->dev_node);
	PVRSRVCommonDeviceDestroy(priv->dev_node);
	mutex_unlock(&g_device_mutex);

	/* dropped
	if (drm_dev->dev->dma_parms == &priv->dma_parms)
		drm_dev->dev->dma_parms = NULL;

	kfree(priv);
	drm_dev->dev_private = NULL;
	*/
	return 0;
}

/**
 * gtgpu_gt69_bind - gengtu gt69-driver initialization function
 * sync from function gt69_probe
 * @dev:
 * parent(dev->parent) is &pci_dev.dev
 * @master: component master
 * @data: point of struct drm_device
 *
 * This function initializes the gengtu gt69 device
 *
 * Returns:
 * 0 if it is OK, errno otherwise.
 */
static int gtgpu_gt69_bind(struct device *dev,
		struct device *master, void *data)
{
	struct drm_device *drm_dev = (struct drm_device *)data;
	struct platform_device *pdev = gt_to_platform_device(dev);
#if defined(NO_HARDWARE)
#else
	plat_data_t *pdata = gt_platform_get_data(pdev);
	pdata->pdev_rsrc->ddev[pdata->dev_idx] = drm_dev;
	if (pdata->pdev_rsrc->pdev == NULL) {
		DRM_ERROR("pdev_rsrc->pdev pci_device is NULL");
	}
#endif
	gt_platform_set_drvdata(pdev, drm_dev);
	/*
	drm_dev->platformdev = pdev; // dropped
	*/
	BUG_ON(!data);
	return gtgpu_gt69_drm_load(dev, 0);
}

/**
 *gtgpu_gt69_unbind - gengtu gt69-driver deinitialization function
 * sync from function gt69_remove
 *@dev:
 *parent(dev->parent) is &pci_dev.dev
 *@master: component master
 *@data: point of struct drm_device
 *
 */
 static void gtgpu_gt69_unbind(struct device *dev,
		struct device *master, void *data)
{
	struct platform_device *pdev = gt_to_platform_device(dev);
	struct drm_device *drm_dev = (struct drm_device *)data;

	BUG_ON(!data);

	gtgpu_gt69_drm_unload(drm_dev);
	gt_platform_set_drvdata(pdev, NULL);
}

/**
 *gtgpu_gt69_unbind - sync from function gt69_init
 *
 */
static int gtgpu_gt69_probe(struct platform_device *pdev)
{
	BUG_ON(!pdev);
	DRM_DEBUG_DRIVER("\n");
	return component_add(&pdev->dev, &s_gtgpu_gt69_ops);
}
#if 0
static void gt69_devices_unregister(void)
{
#if defined(MODULE) && !defined(PVR_LDM_PLATFORM_PRE_REGISTERED)
	unsigned int i;

	BUG_ON(!gt69_devices);

	for (i = 0; i < gt69_num_devices && gt69_devices[i]; i++)
		platform_device_unregister(gt69_devices[i]);

	kfree(gt69_devices);
	gt69_devices = NULL;
#endif /* defined(MODULE) && !defined(PVR_LDM_PLATFORM_PRE_REGISTERED) */
}
#endif
/**
 *gtgpu_gt69_unbind - sync from function gt69_exit
 *
 */
static int gtgpu_gt69_remove(struct platform_device *pdev)
{
	BUG_ON(!pdev);
	component_del(&pdev->dev, &s_gtgpu_gt69_ops);
	return 0;
}


static void gtgpu_gt69_shutdown(struct platform_device *pdev)
{
	struct drm_device *drm_dev = gt_platform_get_drvdata(pdev); //gtgpu_drm_component_bind  gt_platform_set_drvdata(pdev, drm_dev);
	if (!drm_dev) {
		DRM_ERROR("device %p shutdown not bind\n", &pdev->dev);
		return;
	}
	
	DRM_DEBUG_DRIVER("device %p\n", &pdev->dev);

	PVRSRVDeviceShutdown(drm_dev);
}

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3, 9, 0))
static const struct of_device_id gtgpu_gt69_of_ids[] = {
#if defined(SYS_RGX_OF_COMPATIBLE)
	{ .compatible = SYS_RGX_OF_COMPATIBLE, },
#endif
	{},
};

#if !defined(CHROMIUMOS_KERNEL) || !defined(MODULE)
MODULE_DEVICE_TABLE(of, gtgpu_gt69_of_ids);
#endif
#endif

static struct platform_device_id gtgpu_gt69_platform_ids[] = {
#if defined(SYS_RGX_DEV_NAME)
	{ SYS_RGX_DEV_NAME, 0 },
#endif
	{ }
};

#if !defined(CHROMIUMOS_KERNEL) || !defined(MODULE)
MODULE_DEVICE_TABLE(platform, gtgpu_gt69_platform_ids);
#endif

struct platform_driver g_gtgpu_gt69srvkm_driver = {
	.driver = {
		.name		= SYS_RGX_DEV_NAME,
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3, 9, 0))
		.of_match_table	= of_match_ptr(gtgpu_gt69_of_ids),
#endif
		.pm		= &gt69_pm_ops,
	},
	.id_table		= gtgpu_gt69_platform_ids,
	.probe			= gtgpu_gt69_probe,
	.remove			= gtgpu_gt69_remove,
	.shutdown		= gtgpu_gt69_shutdown,
};

MODULE_FIRMWARE("gtgpu/gengt.fw.35.2.1632.23");
MODULE_FIRMWARE("gtgpu/gengt.sh.35.2.1632.23");
MODULE_FIRMWARE("gtgpu/gengt.fw.35.4.1632.23");
MODULE_FIRMWARE("gtgpu/gengt.sh.35.4.1632.23");

MODULE_FIRMWARE("gtgpu/gt6910.fw");
MODULE_FIRMWARE("gtgpu/gt6910.sh");
MODULE_FIRMWARE("gtgpu/gt6908.fw");
MODULE_FIRMWARE("gtgpu/gt6908.sh");

