#include "gtdpu_common.h"
#include "hal_interface.h"
#include "gtdpu_compatibility.h"
#include "gt_debug.h"

#if (DRM_VERSION >= KERNEL_VERSION(5, 19, 0))
#include <drm/display/drm_dp_helper.h>
#else
#include <drm/drm_dp_helper.h>
#endif

#include "gtdpu_dp.h"
#include "gtdpu_dp_common.h"
#include "gtdpu_connector.h"

#define GT_GT6910_DP_NUM  3
#define GT_GT6908_DP_NUM  3
#define GT_GT6912_DP_NUM  3
#define GT_GT6913_DP_NUM 3

struct dp_ext_t {
	gt_waitqueue_head *dp_enc_wq;
	atomic64_t enc_status;
	gt_waitqueue_head *dp_dec_wq;
	atomic64_t dec_status;

	gt_waitqueue_head *dp_aux_wq;
	atomic64_t aux_status;

	spinlock_t intr_slock;

	atomic64_t hpg_status;

	atomic64_t audio_status;

	struct i2c_adapter adapter;
	struct i2c_algo_dp_aux_data algo;

	struct edid *edid;
};

struct dp_ext_t gt6910_dp_ext[GT_GT6910_DP_NUM];
struct dp_ext_t gt6908_dp_ext[GT_GT6908_DP_NUM];
struct dp_ext_t gt6912_dp_ext[GT_GT6912_DP_NUM];
struct dp_ext_t gt6913_dp_ext[GT_GT6913_DP_NUM];

void dp_set_aux_status(struct dp_chip_t *chip, int val)
{
	atomic64_set(&chip->dp_ext->aux_status, val);
}

long long dp_get_aux_status(struct dp_chip_t *chip)
{
	return atomic64_read(&chip->dp_ext->aux_status);
}

void dp_set_hpg_status(struct dp_chip_t *chip, int val)
{
	atomic64_set(&chip->dp_ext->hpg_status, val);
}
long long dp_get_hpg_status(struct dp_chip_t *chip)
{
	return atomic64_read(&chip->dp_ext->hpg_status);
}

void dp_set_audio_status(struct dp_chip_t *chip, int val)
{
	atomic64_set(&chip->dp_ext->audio_status, val);
}
long long dp_get_audio_status(struct dp_chip_t *chip)
{
	return atomic64_read(&chip->dp_ext->audio_status);
}

void *dp_get_edid(struct dp_chip_t *chip)
{
	return chip->dp_ext->edid;
}
void dp_set_edid(struct dp_chip_t *chip, void *val)
{
	chip->dp_ext->edid = (struct edid *)val;
}

int dp_ext_init(struct dp_chip_t *chip)
{
	chip_type_e plat;

	plat = hal_get_chiptype(gt_dev_get_parent(chip->dev));

	switch(plat) {
	case CHIP_GT6910_SOC:
		if(chip->id > GT_GT6910_DP_NUM)
			return -1;
		memset(&gt6910_dp_ext[chip->id], 0, sizeof(struct dp_ext_t));
		chip->dp_ext = &gt6910_dp_ext[chip->id];
		break;
	case CHIP_GT6908_SOC:
		if(chip->id > GT_GT6908_DP_NUM)
			return -1;
		memset(&gt6908_dp_ext[chip->id], 0, sizeof(struct dp_ext_t));
		chip->dp_ext = &gt6908_dp_ext[chip->id];
		break;
	case CHIP_GT6913_SOC:
		if(chip->id > GT_GT6913_DP_NUM)
			return -1;
		memset(&gt6913_dp_ext[chip->id], 0, sizeof(struct dp_ext_t));
		chip->dp_ext = &gt6913_dp_ext[chip->id];
		break;
	case CHIP_GT6912_SOC:
		if(chip->id > GT_GT6912_DP_NUM)
			return -1;
		memset(&gt6912_dp_ext[chip->id], 0, sizeof(struct dp_ext_t));
		chip->dp_ext = &gt6912_dp_ext[chip->id];
		break;
	default:
		gtdpu_err(chip->dev, "dp ext init error");
		return -1;
		break;
	}
	return 0;
}

