/*************************************************************************/ /*!
@File			gtdpu_drm_drv.c
@Title
@Copyright		Copyright (c) Gengtu Technology Ltd. All Rights Reserved
@Description
@License		Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/
#include <linux/platform_device.h>
#include "syscommon.h"
#include "gtdpu_drm_gem.h"
#include "gtdpu_drm_drv.h"
#include "gtdpu_drm_fb.h"
#include "gtdpu_common.h"
#include "gtdpu_dp_common.h"
#include "gtdpu_connector.h"
#include "gtdpu_drm_pm.h"
#include "pdp_drm.h"
#include "gtgpu_drm.h"
#include "kernel_compatibility.h"

#define SWAP_SHMEM_ZEO
#define DPU_BACKUP_ALL_VRAM

unsigned int s_dpu_nums = 0x0f;
module_param(s_dpu_nums, uint, 0444);
MODULE_PARM_DESC(s_dpu_nums, "dpu driver register(default: 0x0f)");

unsigned int s_hdmi_nums = 0x3;
module_param(s_hdmi_nums, uint, 0444);
MODULE_PARM_DESC(s_hdmi_nums,"hdmi driver register(default: 3)");

unsigned int s_dp_nums = 1;
module_param(s_dp_nums, uint, 0444);
MODULE_PARM_DESC(s_dp_nums, "dp driver register(default: 1)");

unsigned int s_lvds_nums = 1;
module_param(s_lvds_nums, uint, 0444);
MODULE_PARM_DESC(s_lvds_nums, "lvds driver register(default: 1)");

unsigned int s_vga_nums = 1;
module_param(s_vga_nums, uint, 0444);
MODULE_PARM_DESC(s_vga_nums, "vga driver register(default: 0)");

unsigned int s_logo_extime = 0;
module_param(s_logo_extime, uint, 0644);
MODULE_PARM_DESC(s_logo_extime, "Controls the minimum execution"
			" time of processes displayed the logo(default: 0s)");

unsigned int s_hw_cursor = 0x00;
module_param(s_hw_cursor, uint, 0444);
MODULE_PARM_DESC(s_hw_cursor, "hw cursor driver register(default: 0x00)");

/*
+ * GT6908
+ * 0-4K+4K
+ *
+ */
unsigned int s_dpu_match = 0x0;
module_param(s_dpu_match, uint, 0444);
MODULE_PARM_DESC(s_dpu_match, "dpu match register(default: 0x00)");

int gtdpu_gem_dumb_create(struct drm_file *drm_file,
		struct drm_device *drm_dev, struct drm_mode_create_dumb *args)
{
	int ret = 0;
	uint32_t flags = 0;
	struct gtdpu_drm_private *dev_priv = NULL;
	gtdpu_shared_mem *pshared_mem = NULL;
	gtdpu_mem_manager *mem_manager = NULL;
	gtdpu_mem_positon pos = 0;
	gtdpu_mem_class class = 0;
	bool visible = 0;

	dev_priv = gtgpu_drm_to_display_private(drm_dev);
	if (!dev_priv) {
		gem_err(drm_dev->dev, "gtdpu dev_priv is NULL.\n");
		return -EINVAL;
	}

	flags = args->flags;
	visible = !((args->flags & DBM_GEM_INVISIBLE) >> 28);;
	pos = gtdpu_gem_dbmflags_to_position(drm_dev, dev_priv, flags);
	class = gtdpu_get_dmbflags_to_class(flags, pos);

	if (class == GTT && !dev_priv->has_gtt_mem) {
		gem_err(drm_dev->dev, "request gtt mem form dpu, but dpu not support\n");
		return -ENOMEM;
	} else if (!visible && !dev_priv->has_inv_mem) {
		gem_err(drm_dev->dev, "request invisible mem form dpu, but dpu not support\n");
		return -ENOMEM;
	}

	/* excute shmem first */
	if (pos == SHMEM_VRAM_POSITION) {
		pshared_mem = &dev_priv->shared_vram_info;
		if ((strncmp(current->comm, "Xorg", 4) == 0) || (strncmp(current->comm, "X", 1) == 0)) {
			gtdpu_xorg_monitor_switch_user(drm_dev, pshared_mem);
		}
		ret = gtdpu_gem_object_create(drm_file, drm_dev,
			pshared_mem->current_user->mem_manager, (void *)args, false);
		if (!ret) {
			return 0;
		}
		pos = VRAM_POSITION;
	}

	if (pos == SYS_GTT_POSITION) {
		mem_manager = dev_priv->gtt_mem_manager;
	} else if (pos == VRAM_POSITION) {
		if (visible) {
			mem_manager = dev_priv->visible_mem_manager;
		} else {
			mem_manager = dev_priv->invisible_mem_manager;
		}
	}
	ret = gtdpu_gem_object_create(drm_file, drm_dev, mem_manager, (void *)args, false);

	if (ret) {
		gem_err(drm_dev->dev, "Visible-%d Input: flags-%#x size-%llu error\n",
				mem_manager->visible, args->flags, args->size);
	}

	return ret;
}

int gtdpu_gem_object_create_ioctl(struct drm_device *drm_dev,
		void *data, struct drm_file *drm_file)
{
	int ret = 0;
	uint32_t flags = 0;
	struct drm_pdp_gem_create *args = data;
	struct gtdpu_drm_private *dev_priv = NULL;
	gtdpu_mem_manager *mem_manager = NULL;
	gtdpu_shared_mem *pshared_mem = NULL;
	gtdpu_mem_positon pos = 0;
	gtdpu_mem_class class = 0;
	bool visible = 0;

	dev_priv = gtgpu_drm_to_display_private(drm_dev);
	if (!dev_priv) {
		gem_err(drm_dev->dev, "gtdpu dev_priv is NULL.\n");
		return -EINVAL;
	}

	flags = args->flags;
	visible = !((args->flags & DBM_GEM_INVISIBLE) >> 28);
	pos = gtdpu_gem_dbmflags_to_position(drm_dev, dev_priv, flags);
	class = gtdpu_get_dmbflags_to_class(flags, pos);

	if (class == GTT && !dev_priv->has_gtt_mem) {
		gem_err(drm_dev->dev, "request gtt mem form dpu, but dpu not support\n");
		return -ENOMEM;
	} else if (!visible && !dev_priv->has_inv_mem) {
		gem_err(drm_dev->dev, "request invisible mem form dpu, but dpu not support\n");
		return -ENOMEM;
	}

	/* excute shmem first */
	if (pos == SHMEM_VRAM_POSITION) {
		pshared_mem = &dev_priv->shared_vram_info;
		if ((strncmp(current->comm, "Xorg", 4)==0) || (strncmp(current->comm, "X", 1)==0)) {
			gtdpu_xorg_monitor_switch_user(drm_dev, pshared_mem);
		}
		ret = gtdpu_gem_object_create(drm_file, drm_dev,
			pshared_mem->current_user->mem_manager, (void *)args, true);
		if (!ret) {
			return 0;
		}
		pos = VRAM_POSITION;
	}

	if (pos == SYS_GTT_POSITION) {
		mem_manager = dev_priv->gtt_mem_manager;
	} else if (pos == VRAM_POSITION) {
		if (visible) {
			mem_manager = dev_priv->visible_mem_manager;
		} else {
			mem_manager = dev_priv->invisible_mem_manager;
		}
	}
	ret = gtdpu_gem_object_create(drm_file, drm_dev, mem_manager, (void *)args, true);
	if (ret) {
		gem_err(drm_dev->dev, "Visible-%d Input: flags-%#x size-%llu error\n",
				mem_manager->visible, args->flags, args->size);
	}

	return ret;
}

int gtdpu_drm_chipinfo_ioctl(struct drm_device *drm_dev,
		void *data, struct drm_file *drm_file)
{
	struct drm_pdp_chip_info *args = data;
	struct gtdpu_drm_private *dev_priv = NULL;

	dev_priv = gtgpu_drm_to_display_private(drm_dev);
	if (!dev_priv) {
		gt_drm_err(drm_dev->dev, "gtdpu dev_priv is NULL.\n");
		return -EINVAL;
	}

	args->chip_type = hal_get_chiptype(drm_dev->dev);
	args->support_inv = hal_has_inv_mem(drm_dev->dev);
	args->support_shmem = dev_priv->has_shared_mem;

	return 0;
}

int gtdpu_drm_gem_addr_ioctl(struct drm_device *drm_dev, void *data,
	struct drm_file *drm_file)
{
	struct drm_pdp_gem_addr *args = (struct drm_pdp_gem_addr*)data;
	struct drm_gem_object *obj = NULL;
	gtdpu_gem_object *gtdpu_obj = NULL;
	uint64_t addr = 0;

	obj = gt_drm_gem_object_lookup(drm_file, args->handle);
	if (obj == NULL) {
		gem_err(drm_dev->dev, "find gem_obj by handle[%d] failed\n", args->handle);
		return -EINVAL;
	}
	gtdpu_obj = to_gtdpu_obj(obj);

	if (args->addr_type == DRM_DEV_PADDR) {
		addr = gtdpu_gem_get_dev_paddr(obj);
	} else if (args->addr_type == DRM_DEV_VADDR) {
		addr = gtdpu_gem_get_dev_vaddr(obj);
	}
	gt_drm_gem_object_put(obj);

	if (addr) {
		args->addr = addr;
		return 0;
	} else {
		gem_err(drm_dev->dev, "invalid handle: %d, invalid addr-type: %d\n",
				args->handle, args->addr_type);
	}

	return -EINVAL;;
}

int gt_gem_object_mmap_ioctl(struct drm_device *drm_dev, void *data,
		struct drm_file *drm_file)
{
	struct drm_pdp_gem_mmap *args = (struct drm_pdp_gem_mmap *)data;

	return drm_gem_dumb_map_offset(drm_file, drm_dev, args->handle, &args->offset);
}

int gt_gem_object_plane_fd_ioctl(struct drm_device *drm_dev,
		void *data, struct drm_file *file)
{
	struct gtdpu_drm_private *dev_priv = NULL;
	struct drm_pdp_base_fd *args = (struct drm_pdp_base_fd *)data;
	struct drm_plane *plane = NULL;
	struct drm_crtc *crtc = NULL;
	struct drm_gem_object *obj = NULL;
#if (DRM_VERSION < KERNEL_VERSION(4, 14, 0))
	struct gt_framebuffer *gtfb = NULL;
#endif
	struct drm_framebuffer *bfb = NULL;
	struct gtdpu_pdp0_drm *pdp0_drm = NULL;
	struct drm_display_mode mode;
	unsigned int vblank_us = 16700;
	int ret = -EINVAL;

	dev_priv = gtgpu_drm_to_display_private(drm_dev);
	if (!dev_priv) {
		dpu_err(drm_dev->dev, "gtdpu dev_priv is NULL.\n");
		return -1;
	}

	if (!s_dpu_support_plane_fd || !dev_priv->drm_nulldisplay) {
		return 0;
	}
	if (args == NULL) {
		return -1;
	}

	drm_for_each_plane(plane, drm_dev) {
		if (plane->base.id == args->plane_id) {
			ret = 0;
			break;
		}
	}
	if (ret) {
		dpu_err(drm_dev->dev, "Not found plane id %d\n", args->plane_id);
		return ret;
	}

	memset(&mode, 0, sizeof(mode));
	if (plane->state && plane->state->crtc) {
		crtc = plane->state->crtc;
		mode = crtc->state->mode;
	} else if (!plane->state && plane->crtc) {
		crtc = plane->crtc;
		mode = crtc->mode;
	} else {
		dpu_err(drm_dev->dev, "Plane %d not crtc\n", args->plane_id);
		return -EIO;
	}

	/* calc wait time us, by frame_id and frame rate */
	pdp0_drm = crtc_to_pdp0_device(crtc);
	if (mode.clock) {
		vblank_us = 1000000 / drm_mode_vrefresh(&mode);
	}

	if (args->frame_id > atomic64_read(&pdp0_drm->active_count)) {
		vblank_us *= (atomic64_read(&pdp0_drm->active_count) - args->frame_id);
	} else {
		/* at lease wait next active region comming */
		args->frame_id = atomic64_read(&pdp0_drm->active_count) + 1;
	}

	/* wait target active region access */
	pdp0_drm = crtc_to_pdp0_device(crtc);
	if (pdp0_drm->active_wq != NULL) {
		ret = gt_wait_event_interruptible_timeout(pdp0_drm->active_wq,
			atomic64_read(&pdp0_drm->active_count) >= args->frame_id,
			usecs_to_jiffies(vblank_us + 100000)); /* add 100ms as margin */
		if (ret < 0) {
			dpu_err(drm_dev->dev, "wait %d gengtu failed\n", args->plane_id);
			return ret;
		} else if (ret == 0) {
			dpu_err(drm_dev->dev, "wait %d gengtu valid timeout\n", args->plane_id);
			return -ETIMEDOUT;
		}
	}

	/* get bfb here */
	if (plane->state && plane->state->fb) {
		bfb = plane->state->fb;
	} else if (!plane->state && plane->fb) {
		bfb = plane->fb;
	} else {
		dpu_err(drm_dev->dev, "Plane %d has not fb\n", args->plane_id);
		return -EIO;
	}

	ret = 0;

	/* plane obj */
#if (DRM_VERSION < KERNEL_VERSION(4, 14, 0))
	gtfb = to_gt_framebuffer(bfb);
	obj = gtfb->obj[0];
#else
	obj = bfb->obj[0];
#endif

#if (DRM_VERSION >= KERNEL_VERSION(4, 13, 0))
	args->format = bfb->format->format;
#else
	args->format = bfb->pixel_format;
#endif
	args->frame_size = obj->size;
	args->width = bfb->width;
	args->height = bfb->height;
	args->obj_idr = obj->name;
	args->frame_id = atomic64_read(&pdp0_drm->active_count);

	/* todo: support cursor_obj */
	args->cursor_obj_idr = 0;
	args->cursor_x = 0;
	args->cursor_y = 0;
	args->cursor_width = 0;
	args->cursor_height = 0;
	args->cursor_format = 0;

	if (args->obj_idr <= 0) {
		dpu_err(drm_dev->dev, "unknow err of get plane[%d] obj_name[%d]\n",
				args->plane_id, args->obj_idr);
	}

	return ret;
}


#ifndef CONFIG_VIDEOMODE_HELPERS
/**
 * drm_display_mode_to_videomode - fill in @vm using @dmode,
 * @dmode: drm_display_mode structure to use as source
 * @vm: videomode structure to use as destination
 *
 * Fills out @vm using the display mode specified in @dmode.
 */
void drm_display_mode_to_videomode(
		const struct drm_display_mode *dmode,struct videomode *vm)
{
	vm->hactive = dmode->hdisplay;
	vm->hfront_porch = dmode->hsync_start - dmode->hdisplay;
	vm->hsync_len = dmode->hsync_end - dmode->hsync_start;
	vm->hback_porch = dmode->htotal - dmode->hsync_end;

	vm->vactive = dmode->vdisplay;
	vm->vfront_porch = dmode->vsync_start - dmode->vdisplay;
	vm->vsync_len = dmode->vsync_end - dmode->vsync_start;
	vm->vback_porch = dmode->vtotal - dmode->vsync_end;

	vm->pixelclock = dmode->clock * 1000;

	vm->flags = 0;
	if (dmode->flags & DRM_MODE_FLAG_PHSYNC)
		vm->flags |= DISPLAY_FLAGS_HSYNC_HIGH;
	else if (dmode->flags & DRM_MODE_FLAG_NHSYNC)
		vm->flags |= DISPLAY_FLAGS_HSYNC_LOW;
	if (dmode->flags & DRM_MODE_FLAG_PVSYNC)
		vm->flags |= DISPLAY_FLAGS_VSYNC_HIGH;
	else if (dmode->flags & DRM_MODE_FLAG_NVSYNC)
		vm->flags |= DISPLAY_FLAGS_VSYNC_LOW;
	if (dmode->flags & DRM_MODE_FLAG_INTERLACE)
		vm->flags |= DISPLAY_FLAGS_INTERLACED;
	if (dmode->flags & DRM_MODE_FLAG_DBLSCAN)
		vm->flags |= DISPLAY_FLAGS_DOUBLESCAN;
	if (dmode->flags & DRM_MODE_FLAG_DBLCLK)
		vm->flags |= DISPLAY_FLAGS_DOUBLECLK;
}
#endif

#if (DRM_VERSION >= KERNEL_VERSION(6, 0, 0))

#include <linux/aperture.h>
int drm_fb_helper_remove_conflicting_framebuffers(const char *name)
{
#if (DRM_VERSION >= KERNEL_VERSION(6, 5, 0))
	aperture_remove_all_conflicting_devices(name);
#else
	aperture_remove_all_conflicting_devices(false, name);
#endif
	return 0;
}

#elif (DRM_VERSION >= KERNEL_VERSION(5, 14, 0))

int drm_fb_helper_remove_conflicting_framebuffers(struct apertures_struct *a,
		const char *name, bool primary)
{
#if IS_REACHABLE(CONFIG_FB)
	return remove_conflicting_framebuffers(a, name, primary);
#else
	return 0;
#endif
}

#endif

static void gtdpu_kick_off_efifb(struct drm_device *drm_dev)
{
#if (DRM_VERSION < KERNEL_VERSION(6, 0, 0))
	struct apertures_struct *a;

	a = alloc_apertures(1);
	if (!a) {
		gt_drm_err(drm_dev->dev, "Short of memory,gtdrmfb will not take over fbcon !\n");
		return;
	}

	/* method 1: remove gt_efifb */
	//a->ranges[0].base = hal_get_ddrbase(drm_dev->dev);
	//a->ranges[0].size = hal_get_ddr_bar_len(drm_dev->dev);

	/* method 2: remove all fb_dev */
	a->ranges[0].base = 0;
	a->ranges[0].size = ~0;

	drm_fb_helper_remove_conflicting_framebuffers(a, "gtdrmfb", true);

	kfree(a);
#else
	drm_fb_helper_remove_conflicting_framebuffers("gtdrmfb");
#endif
}

int gtdpu_fb_init(struct drm_device *drm_dev,
		struct gtdpu_drm_private *dev_priv)
{
	int ret = 0;

	if (!dev_priv->drm_nulldisplay) {
		gt_drm_info(drm_dev->dev, "remove efifb...");
		gtdpu_kick_off_efifb(drm_dev);
		gt_drm_info(drm_dev->dev, "Start init fbdev device");
		ret = gtdpu_fbdev_init(drm_dev);
		if (ret) {
			gt_drm_err(drm_dev->dev, "Failed to Init fbdev(%d)", ret);
			return ret;
		}
	}

	return 0;
}

void gtdpu_fb_fini(struct drm_device *drm_dev,
		struct gtdpu_drm_private *dev_priv)
{
	if (!dev_priv->drm_nulldisplay) {
		gtdpu_fbdev_fini(drm_dev);
	}
}

int gtdpu_pm_init(struct drm_device *drm_dev,
		struct gtdpu_drm_private *dev_priv)
{
	if (!dev_priv->drm_nulldisplay) {
		dev_priv->pm_nb.notifier_call = gtdpu_pm_notifier;
		register_pm_notifier(&dev_priv->pm_nb);
	}

	return 0;
}

void gtdpu_pm_fini(struct drm_device *drm_dev,
		struct gtdpu_drm_private *dev_priv)
{
	if (!dev_priv->drm_nulldisplay) {
		unregister_pm_notifier(&dev_priv->pm_nb);
	}
}

static int gtdpu_chip_init(struct drm_device *drm_dev,
		struct gtdpu_drm_private *dev_priv)
{
	return gtdpu_custom_init(drm_dev, dev_priv);
}

int gtdpu_early_load(struct drm_device *drm_dev,
		struct gtdpu_drm_private *dev_priv, struct device *dev)
{
	int err = 0;
	dev_priv->dev = dev;
	dev_priv->drm_dev = drm_dev;
	dev_priv->display_enabled = true;
	dev_priv->has_shared_mem = false;
	dev_priv->has_gtt_mem = false;
	dev_priv->has_pdp_vga_mem = false;
	dev_priv->has_uncontinuous_mem = false;
	dev_priv->logo_end_ktime = 0;

	gt_drm_info(dev, "Start early load gengtu drm device");

	/* nulldisp info */
	err  = gtgpu_drm_get_nulldisp(drm_dev, &dev_priv->drm_nulldisplay);
	if (err < 0) {
		gt_drm_err(dev, "Get nulldisp failed(%d)", err);
		return err;
	}

	/* sharemem info */
	if (!dev_priv->drm_nulldisplay && s_dpu_has_shared_mem) {
		dev_priv->has_shared_mem = true;
		dev_priv->shared_vram_info.size = s_dpu_shared_mem_size;
	}

	/* inv info */
	err = hal_has_inv_mem(drm_dev->dev);
	if (err) {
		dev_priv->has_inv_mem = true;
	}

	/* gtt info */
	err = hal_has_gtt_mem(drm_dev->dev);
	if (err && s_dpu_has_gtt_mem) {
		dev_priv->has_gtt_mem = true;
	}

	/* no continuous vram */
	if (s_dpu_has_nocontinuous_vram) {
		dev_priv->has_uncontinuous_mem = true;
	}

	/* pdp for vga info */
	if (s_vga_auto_adapt && !dev_priv->drm_nulldisplay) {
		dev_priv->has_pdp_vga_mem = true;
	}

	mutex_init(&dev_priv->init_lock);
	mutex_init(&dev_priv->logo_lock);

	err = gtdpu_modeset_early_init(drm_dev, dev_priv);
	if (err) {
		gt_drm_err(dev, "Early modeset initialisation failed(%d)", err);
		goto err_modeset_early_init;
	}

	gtdpu_chip_init(drm_dev, dev_priv);

	gt_drm_info(dev, "has_shared_mem = %d, has_inv_mem = %d",
					  dev_priv->has_shared_mem, dev_priv->has_inv_mem);
	gt_drm_info(dev, "hal_gtt_mem = %d, has_uncontinuous_mem = %d, has_pdp_vga_mem = %d",
						dev_priv->has_gtt_mem, dev_priv->has_uncontinuous_mem,
						dev_priv->has_pdp_vga_mem);
	gt_drm_info(dev, "Early modeset initialization completed");

	return 0;

err_modeset_early_init:
	return err;
}

void gtdpu_early_unload(struct drm_device *drm_dev,
		struct gtdpu_drm_private *dev_priv)
{
#if (DRM_VERSION < KERNEL_VERSION(4, 14, 0))
	/* Called by drm_dev_fini in Linux 4.11.0 and later */
	drm_vblank_cleanup(drm_dev);
#endif
}

static void gtdpu_vram_fini(struct drm_device *drm_dev,
		struct gtdpu_drm_private *dev_priv)
{
	if (dev_priv->zero_gem)
		gtdpu_zero_mem_fini(drm_dev, dev_priv->zero_gem);
	if (dev_priv->pdp_vga_gem && dev_priv->has_pdp_vga_mem)
		gtdpu_pdp_vga_mem_fini(drm_dev, dev_priv->pdp_vga_gem);
	if (dev_priv->visible_mem_manager)
		gtdpu_mem_manager_fini(drm_dev, dev_priv->visible_mem_manager);
	if (dev_priv->invisible_mem_manager && dev_priv->has_inv_mem)
		gtdpu_mem_manager_fini(drm_dev, dev_priv->invisible_mem_manager);
	if (dev_priv->gtt_mem_manager && dev_priv->has_gtt_mem)
		gtdpu_mem_manager_fini(drm_dev, dev_priv->gtt_mem_manager);

	if (dev_priv->shared_vram_info.dev_paddr != 0 && dev_priv->has_shared_mem) {
		idr_destroy(&dev_priv->shared_vram_info.uer_idr);
		gtdpu_gem_share_pre_free(drm_dev, &dev_priv->shared_vram_info);
	}
}

static int gtdpu_vram_init(struct drm_device *drm_dev,
		struct gtdpu_drm_private *dev_priv)
{
	int err = 0;

	gem_info(drm_dev->dev, "vram init, display mode is %s, shmem is %s",
			 dev_priv->drm_nulldisplay ? "nulldisplay" : "display",
			 s_dpu_has_shared_mem ? "enabled" : "disabled");

	dev_priv->role.vram_role = HAL_VRAM_ROLE_DPU;
	dev_priv->role.id = dev_priv->role_id;
	dev_priv->role.sub_id = 0;

	/* zero vram init */
	dev_priv->zero_gem = gtdpu_zero_mem_init(drm_dev, GEM_ZERO_SIZE);
	if (!dev_priv->zero_gem) {
		gem_err(drm_dev->dev, "GT zero mem init failed. Short of memory");
		err = -ENOMEM;
		goto err_gem_init;
	}

	/* pdp vga mem init */
	if (dev_priv->has_pdp_vga_mem) {
		dev_priv->pdp_vga_gem = gtdpu_pdp_vga_mem_init(drm_dev);
		if (!dev_priv->pdp_vga_gem) {
			gem_err(drm_dev->dev, "GT pdp vga gem init failed. Short of memory");
			err = -ENOMEM;
			goto err_gem_init;
		}
	}

	/* visible mem */
	dev_priv->visible_mem_manager = gtdpu_mem_manager_init(drm_dev, true, VRAM_POSITION, NULL);
	if (!dev_priv->visible_mem_manager) {
		gem_err(drm_dev->dev, "GT visible gem init failed. Short of memory");
		err = -ENOMEM;
		goto err_gem_init;
	}

	/* invisible mem */
	if (dev_priv->has_inv_mem) {
		dev_priv->invisible_mem_manager = gtdpu_mem_manager_init(drm_dev, false, VRAM_POSITION, NULL);
		if (!dev_priv->invisible_mem_manager) {
			gem_err(drm_dev->dev, "GT invisible gem init failed. Short of memory");
			err = -ENOMEM;
			goto err_gem_init;
		}
	}

	/* gtt mem init */
	if (dev_priv->has_gtt_mem) {
		dev_priv->gtt_mem_manager = gtdpu_mem_manager_init(drm_dev, true, SYS_GTT_POSITION, NULL);
		if (!dev_priv->gtt_mem_manager) {
			gem_err(drm_dev->dev, "GT gtt gem init failed. Short of memory");
			err = - EINVAL;
			goto err_gem_init;
		}
	}

	/* shared mem init */
	if (dev_priv->has_shared_mem) {

		if (dev_priv->has_inv_mem) {
			dev_priv->shared_vram_info.is_visible = false;
		}

		err = gtdpu_gem_share_pre_alloc(drm_dev, &dev_priv->shared_vram_info,
					s_dpu_shared_mem_size / hal_get_gpu_core_nums());
		if (err < 0) {
			goto err_gem_init;
		}

		idr_init_base(&dev_priv->shared_vram_info.uer_idr, 0);
		spin_lock_init(&dev_priv->shared_vram_info.user_idr_lock);
		dev_priv->shared_vram_info.current_user = NULL;
	}

	return err;

err_gem_init:
	gtdpu_vram_fini(drm_dev, dev_priv);

	return err;
}

int gtdpu_late_load(struct drm_device *drm_dev,
		struct gtdpu_drm_private *dev_priv)
{
	int err = 0;

	gt_drm_info(drm_dev->dev, "Start late load gengtu drm device");

	err = gtdpu_vram_init(drm_dev, dev_priv);
	if (err < 0) {
		gt_drm_err(drm_dev->dev, "gt vram init failed(%d)", err);
		goto err_vram_init;
	}

	// drm_vblank_init：vblank自动释放 (drm_vblank_init_release)
	err = drm_vblank_init(drm_dev, drm_dev->mode_config.num_crtc);
	if (err < 0) {
		gt_drm_err(drm_dev->dev, "Failed to complete vblank init(%d)", err);
		goto err_vblank_init;
	}
#if (DRM_VERSION < KERNEL_VERSION(4, 7, 0))
	drm_dev->vblank_disable_allowed = 1;
#endif
	gt_drm_info(drm_dev->dev, "drm vblank init with crtc nums = %d, connector = %d",
				  drm_dev->mode_config.num_crtc, drm_dev->mode_config.num_connector);

	err = gtdpu_modeset_late_init(drm_dev, dev_priv);
	if (err) {
		gt_drm_err(drm_dev->dev, "late modeset initialisation failed(%d)", err);
		goto err_modeset_late_init;
	}

	return err;

err_modeset_late_init:
#if (DRM_VERSION < KERNEL_VERSION(4, 7, 0))
	drm_dev->vblank_disable_allowed = 0;
#endif

err_vblank_init:
	gtdpu_vram_fini(drm_dev, dev_priv);

err_vram_init:
	return err;
}

void gtdpu_late_unload(struct drm_device *drm_dev,
		struct gtdpu_drm_private *dev_priv)
{
	drm_kms_helper_poll_fini(drm_dev);
	drm_mode_config_cleanup(drm_dev);
	gtdpu_vram_fini(drm_dev, dev_priv);
}

void gtdpu_bind_atomic(struct drm_device * drm_dev, bool lock)
{
	struct gtdpu_drm_private *dev_priv = NULL;

	dev_priv = gtgpu_drm_to_display_private(drm_dev);
	if (!dev_priv) {
		gt_drm_err(drm_dev->dev, "gtdpu_drm_private does not find");
		return;
	}

	if (lock) {
		mutex_lock(&dev_priv->init_lock);
	} else {
		mutex_unlock(&dev_priv->init_lock);
	}
}
