#include "gtdpu_common.h"
#include "hal_interface.h"
#include "gtdpu_compatibility.h"
#include "gt_debug.h"

#if (DRM_VERSION >= KERNEL_VERSION(5, 19, 0))
#include <drm/display/drm_dp_helper.h>
#else
#include <drm/drm_dp_helper.h>
#endif

#include "gtdpu_hdmi.h"
#include "gtdpu_hdmi_common.h"
#include "gtdpu_hdmi_debugfs.h"
#include "gtdpu_connector.h"

#define GT_GT6910_HDMI_EXT_MAX  3
#define GT_GT6912_HDMI_EXT_MAX  3
#define GT_GT6913_HDMI_EXT_MAX 3
#define GT_GT6908_HDMI_EXT_MAX  3


struct hdmi_ext_t gt6910_hdmi_ext[GT_GT6910_HDMI_EXT_MAX];
struct hdmi_ext_t gt6908_hdmi_ext[GT_GT6908_HDMI_EXT_MAX];
struct hdmi_ext_t gt6912_hdmi_ext[GT_GT6912_HDMI_EXT_MAX];
struct hdmi_ext_t gt6913_hdmi_ext[GT_GT6913_HDMI_EXT_MAX];

int gt6910_soc_hdmi_debugfs_edidparse_init(struct hdmi_chip_t *chip)
{
	struct hdmi_debugfs_item_t *edid_info;

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_edidparse.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_edidparse.debugfs_lock);

	edid_info = (struct hdmi_debugfs_item_t *)kmalloc(
							sizeof(struct hdmi_debugfs_item_t), gt_gfp_kernel);
	edid_info->chip = chip;
	edid_info->ent = NULL;
	edid_info->status = HDMI_DEBUGS_STATUS_UNINITED;
	edid_info->data = "edid_info";

	mutex_lock(&chip->hdmi_ext->custom_edidparse.debugfs_lock);
	list_add(&edid_info->list, &chip->hdmi_ext->custom_edidparse.debugfs_list);
	mutex_unlock(&chip->hdmi_ext->custom_edidparse.debugfs_lock);

	return 0;
}

int gt6910_soc_hdmi_debugfs_bisttest_init(struct hdmi_chip_t *chip)
{
	struct hdmi_debugfs_item_t *bisttest;

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_bisttest.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_bisttest.debugfs_lock);

	bisttest = (struct hdmi_debugfs_item_t *)kmalloc(
							sizeof(struct hdmi_debugfs_item_t), gt_gfp_kernel);
	bisttest->chip = chip;
	bisttest->ent = NULL;
	bisttest->status = HDMI_DEBUGS_STATUS_UNINITED;
	bisttest->data = "bisttest";

	mutex_lock(&chip->hdmi_ext->custom_bisttest.debugfs_lock);
	list_add(&bisttest->list, &chip->hdmi_ext->custom_bisttest.debugfs_list);
	mutex_unlock(&chip->hdmi_ext->custom_bisttest.debugfs_lock);

	return 0;
}

int gt6910_soc_hdmi_debugfs_hdmi_status_init(struct hdmi_chip_t *chip)
{
	struct hdmi_debugfs_item_t *hdmi_status;

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_hdmi_status.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_hdmi_status.debugfs_lock);

	hdmi_status = (struct hdmi_debugfs_item_t *)kmalloc(
							sizeof(struct hdmi_debugfs_item_t), gt_gfp_kernel);
	hdmi_status->chip = chip;
	hdmi_status->ent = NULL;
	hdmi_status->status = HDMI_DEBUGS_STATUS_UNINITED;
	hdmi_status->data = "gt_status";

	mutex_lock(&chip->hdmi_ext->custom_hdmi_status.debugfs_lock);
	list_add(&hdmi_status->list, &chip->hdmi_ext->custom_hdmi_status.debugfs_list);
	mutex_unlock(&chip->hdmi_ext->custom_hdmi_status.debugfs_lock);

	return 0;
}

int gt6910_soc_hdmi_debugfs_hw_test_init(struct hdmi_chip_t *chip)
{
	struct hdmi_debugfs_item_t *hw_test;

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_hw_self_test.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_hw_self_test.debugfs_lock);

	hw_test = (struct hdmi_debugfs_item_t *)kmalloc(
					sizeof(struct hdmi_debugfs_item_t), gt_gfp_kernel);
	hw_test->chip = chip;
	hw_test->ent = NULL;
	hw_test->status = HDMI_DEBUGS_STATUS_UNINITED;
	hw_test->data = "hw_self_test";

	mutex_lock(&chip->hdmi_ext->custom_hw_self_test.debugfs_lock);
	list_add(&hw_test->list, &chip->hdmi_ext->custom_hw_self_test.debugfs_list);
	mutex_unlock(&chip->hdmi_ext->custom_hw_self_test.debugfs_lock);

	return 0;
}

int gt6913_soc_hdmi_debugfs_edidparse_init(struct hdmi_chip_t *chip)
{
	struct hdmi_debugfs_item_t *edid_info;

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_edidparse.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_edidparse.debugfs_lock);

	edid_info = (struct hdmi_debugfs_item_t *)kmalloc(
							sizeof(struct hdmi_debugfs_item_t), gt_gfp_kernel);
	edid_info->chip = chip;
	edid_info->ent = NULL;
	edid_info->status = HDMI_DEBUGS_STATUS_UNINITED;
	edid_info->data = "edid_info";

	mutex_lock(&chip->hdmi_ext->custom_edidparse.debugfs_lock);
	list_add(&edid_info->list, &chip->hdmi_ext->custom_edidparse.debugfs_list);
	mutex_unlock(&chip->hdmi_ext->custom_edidparse.debugfs_lock);

	return 0;
}

int gt6913_soc_hdmi_debugfs_bisttest_init(struct hdmi_chip_t *chip)
{
	struct hdmi_debugfs_item_t *bisttest;

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_bisttest.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_bisttest.debugfs_lock);

	bisttest = (struct hdmi_debugfs_item_t *)kmalloc(
							sizeof(struct hdmi_debugfs_item_t), gt_gfp_kernel);
	bisttest->chip = chip;
	bisttest->ent = NULL;
	bisttest->status = HDMI_DEBUGS_STATUS_UNINITED;
	bisttest->data = "bisttest";

	mutex_lock(&chip->hdmi_ext->custom_bisttest.debugfs_lock);
	list_add(&bisttest->list, &chip->hdmi_ext->custom_bisttest.debugfs_list);
	mutex_unlock(&chip->hdmi_ext->custom_bisttest.debugfs_lock);

	return 0;
}

int gt6913_soc_hdmi_debugfs_hw_test_init(struct hdmi_chip_t *chip)
{
	struct hdmi_debugfs_item_t *hw_test;

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_hw_self_test.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_hw_self_test.debugfs_lock);

	hw_test = (struct hdmi_debugfs_item_t *)kmalloc(
			sizeof(struct hdmi_debugfs_item_t), gt_gfp_kernel);
	hw_test->chip = chip;
	hw_test->ent = NULL;
	hw_test->status = HDMI_DEBUGS_STATUS_UNINITED;
	hw_test->data = "hw_self_test";

	mutex_lock(&chip->hdmi_ext->custom_hw_self_test.debugfs_lock);
	list_add(&hw_test->list, &chip->hdmi_ext->custom_hw_self_test.debugfs_list);
	mutex_unlock(&chip->hdmi_ext->custom_hw_self_test.debugfs_lock);

	return 0;
}

int gt6913_soc_hdmi_debugfs_hdmi_status_init(struct hdmi_chip_t *chip)
{
	struct hdmi_debugfs_item_t *hdmi_status;

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_hdmi_status.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_hdmi_status.debugfs_lock);

	hdmi_status = (struct hdmi_debugfs_item_t *)kmalloc(sizeof(struct hdmi_debugfs_item_t), gt_gfp_kernel);
	hdmi_status->chip = chip;
	hdmi_status->ent = NULL;
	hdmi_status->status = HDMI_DEBUGS_STATUS_UNINITED;
	hdmi_status->data = "gt_status";

	mutex_lock(&chip->hdmi_ext->custom_hdmi_status.debugfs_lock);
	list_add(&hdmi_status->list, &chip->hdmi_ext->custom_hdmi_status.debugfs_list);
	mutex_unlock(&chip->hdmi_ext->custom_hdmi_status.debugfs_lock);

	return 0;
}

int gt6912_soc_hdmi_debugfs_bisttest_init(struct hdmi_chip_t *chip)
{

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_bisttest.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_bisttest.debugfs_lock);

	return 0;
}

int gt6912_soc_hdmi_debugfs_edidparse_init(struct hdmi_chip_t *chip)
{
	INIT_LIST_HEAD(&chip->hdmi_ext->custom_edidparse.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_edidparse.debugfs_lock);

	return 0;
}

int gt6912_soc_hdmi_debugfs_hw_test_init(struct hdmi_chip_t *chip)
{
	struct hdmi_debugfs_item_t *hw_test;

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_hw_self_test.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_hw_self_test.debugfs_lock);

	hw_test = (struct hdmi_debugfs_item_t *)kmalloc(
					sizeof(struct hdmi_debugfs_item_t), gt_gfp_kernel);
	hw_test->chip = chip;
	hw_test->ent = NULL;
	hw_test->status = HDMI_DEBUGS_STATUS_UNINITED;
	hw_test->data = "hw_self_test";

	mutex_lock(&chip->hdmi_ext->custom_hw_self_test.debugfs_lock);
	list_add(&hw_test->list, &chip->hdmi_ext->custom_hw_self_test.debugfs_list);
	mutex_unlock(&chip->hdmi_ext->custom_hw_self_test.debugfs_lock);

	return 0;
}

int gt6912_soc_hdmi_debugfs_hdmi_status_init(struct hdmi_chip_t *chip)
{
	INIT_LIST_HEAD(&chip->hdmi_ext->custom_hdmi_status.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_hdmi_status.debugfs_lock);

	return 0;
}

void hdmi_set_hpg_status(struct hdmi_chip_t *chip, int val)
{
	atomic64_set(&chip->hdmi_ext->hpg_status, val);
}

long long hdmi_get_hpg_status(struct hdmi_chip_t *chip)
{
	return atomic64_read(&chip->hdmi_ext->hpg_status);
}

int hdmi_ext_init(struct hdmi_chip_t *chip)
{
	chip_type_e plat;

	plat = hal_get_chiptype(gt_dev_get_parent(chip->dev));

	switch(plat) {
	case CHIP_GT6910_SOC:
		if(chip->id > GT_GT6910_HDMI_EXT_MAX)
			return -1;
		memset(&gt6910_hdmi_ext[chip->id], 0, sizeof(struct hdmi_ext_t));
		chip->hdmi_ext = &gt6910_hdmi_ext[chip->id];
		break;
	case CHIP_GT6912_SOC:
		if(chip->id > GT_GT6912_HDMI_EXT_MAX)
			return -1;
		memset(&gt6912_hdmi_ext[chip->id], 0, sizeof(struct hdmi_ext_t));
		chip->hdmi_ext = &gt6912_hdmi_ext[chip->id];
		break;
	case CHIP_GT6913_SOC:
		if(chip->id > GT_GT6913_HDMI_EXT_MAX)
			return -1;
		memset(&gt6913_hdmi_ext[chip->id], 0, sizeof(struct hdmi_ext_t));
		chip->hdmi_ext = &gt6913_hdmi_ext[chip->id];
		break;
	case CHIP_GT6908_SOC:
		if(chip->id > GT_GT6908_HDMI_EXT_MAX)
			return -1;
		memset(&gt6908_hdmi_ext[chip->id], 0, sizeof(struct hdmi_ext_t));
		chip->hdmi_ext = &gt6908_hdmi_ext[chip->id];
		break;
	default:
		gtdpu_err(chip->dev, "hdmi ext init error");
		return -1;
		break;
	}
	return 0;
}

int gt6908_soc_hdmi_debugfs_edidparse_init(struct hdmi_chip_t *chip)
{
	struct hdmi_debugfs_item_t *edid_info;

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_edidparse.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_edidparse.debugfs_lock);

	edid_info = (struct hdmi_debugfs_item_t *)kmalloc(
							sizeof(struct hdmi_debugfs_item_t), gt_gfp_kernel);
	edid_info->chip = chip;
	edid_info->ent = NULL;
	edid_info->status = HDMI_DEBUGS_STATUS_UNINITED;
	edid_info->data = "edid_info";

	mutex_lock(&chip->hdmi_ext->custom_edidparse.debugfs_lock);
	list_add(&edid_info->list, &chip->hdmi_ext->custom_edidparse.debugfs_list);
	mutex_unlock(&chip->hdmi_ext->custom_edidparse.debugfs_lock);

	return 0;
}

int gt6908_soc_hdmi_debugfs_bisttest_init(struct hdmi_chip_t *chip)
{
	struct hdmi_debugfs_item_t *bisttest;

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_bisttest.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_bisttest.debugfs_lock);

	bisttest = (struct hdmi_debugfs_item_t *)kmalloc(
							sizeof(struct hdmi_debugfs_item_t), gt_gfp_kernel);
	bisttest->chip = chip;
	bisttest->ent = NULL;
	bisttest->status = HDMI_DEBUGS_STATUS_UNINITED;
	bisttest->data = "bisttest";

	mutex_lock(&chip->hdmi_ext->custom_bisttest.debugfs_lock);
	list_add(&bisttest->list, &chip->hdmi_ext->custom_bisttest.debugfs_list);
	mutex_unlock(&chip->hdmi_ext->custom_bisttest.debugfs_lock);

	return 0;
}

int gt6908_soc_hdmi_debugfs_hdmi_status_init(struct hdmi_chip_t *chip)
{
	struct hdmi_debugfs_item_t *hdmi_status;

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_hdmi_status.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_hdmi_status.debugfs_lock);

	hdmi_status = (struct hdmi_debugfs_item_t *)kmalloc(
							sizeof(struct hdmi_debugfs_item_t), gt_gfp_kernel);
	hdmi_status->chip = chip;
	hdmi_status->ent = NULL;
	hdmi_status->status = HDMI_DEBUGS_STATUS_UNINITED;
	hdmi_status->data = "gt_status";

	mutex_lock(&chip->hdmi_ext->custom_hdmi_status.debugfs_lock);
	list_add(&hdmi_status->list, &chip->hdmi_ext->custom_hdmi_status.debugfs_list);
	mutex_unlock(&chip->hdmi_ext->custom_hdmi_status.debugfs_lock);

	return 0;
}

int gt6908_soc_hdmi_debugfs_hw_test_init(struct hdmi_chip_t *chip)
{
	struct hdmi_debugfs_item_t *hw_test;

	INIT_LIST_HEAD(&chip->hdmi_ext->custom_hw_self_test.debugfs_list);
	mutex_init(&chip->hdmi_ext->custom_hw_self_test.debugfs_lock);

	hw_test = (struct hdmi_debugfs_item_t *)kmalloc(
					sizeof(struct hdmi_debugfs_item_t), gt_gfp_kernel);
	hw_test->chip = chip;
	hw_test->ent = NULL;
	hw_test->status = HDMI_DEBUGS_STATUS_UNINITED;
	hw_test->data = "hw_self_test";

	mutex_lock(&chip->hdmi_ext->custom_hw_self_test.debugfs_lock);
	list_add(&hw_test->list, &chip->hdmi_ext->custom_hw_self_test.debugfs_list);
	mutex_unlock(&chip->hdmi_ext->custom_hw_self_test.debugfs_lock);

	return 0;
}

void *gt_hdmi_get_chip_adapater(struct hdmi_chip_t *chip)
{
	struct hdmi_device_t *gt_hdmi
		= container_of(chip, struct hdmi_device_t, chip);

	return &gt_hdmi->i2c->adapter;
}

