/*
* Copyright (c) Gengtu Technology Ltd. All Rights Reserved
* Dual MIT/GPLv2
*
* The contents of this file are subject to the MIT license as set out below.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Alternatively, the contents of this file may be used under the terms of
* the GNU General Public License Version 2 ("GPL") in which case the provisions
* of GPL are applicable instead of those above.
*
* If you wish to allow use of your version of this file only under the terms of
* GPL, and not to allow others to use your version of this file under the terms
* of the MIT license, indicate your decision by deleting the provisions above
* and replace them with the notice and other provisions required by GPL as set
* out in the file called "GPL-COPYING" included in this distribution. If you do
* not delete the provisions above, a recipient may use your version of this file
* under the terms of either the MIT license or GPL.
*
* This License is also included in this distribution in the file called
* "MIT-COPYING".
*
* EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
* PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
* BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
* PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#ifndef __GT_DMA_BUF_H__
#define __GT_DMA_BUF_H__

#include "gt_mm.h"
#include "gt_dma.h"
#include <linux/types.h>

typedef void gt_dma_buf;
typedef void gt_dma_buf_attachment;

int gt_dma_buf_begin_cpu_access(gt_dma_buf *dmabuf, enum gt_dma_data_direction dir);

int gt_dma_buf_end_cpu_access(gt_dma_buf *dmabuf, enum gt_dma_data_direction dir);

void *gt_dma_buf_vmap(gt_dma_buf *dmabuf);

void gt_dma_buf_vunmap(gt_dma_buf *dmabuf, void *addr);

void *gt_dma_buf_kmap(gt_dma_buf *dmabuf, unsigned long page_num);

void gt_dma_buf_kunmap(gt_dma_buf *dmabuf, unsigned long page_num, void *addr);

int gt_dma_buf_mmap(gt_dma_buf *dmabuf, gt_vm_area *vma, unsigned long pgoff);

gt_dma_buf_attachment *gt_dma_buf_attach(gt_dma_buf *dmabuf, void *dev);

gt_dev* gt_dma_buf_attach_get_dev(gt_dma_buf_attachment *attach);

void gt_dma_buf_detach(gt_dma_buf *dmabuf, gt_dma_buf_attachment *attach);

gt_sg_table *gt_dma_buf_map_attachment(gt_dma_buf_attachment *attach, enum gt_dma_data_direction dir);

void gt_dma_buf_unmap_attachment(gt_dma_buf_attachment *attach, gt_sg_table *sg_table, enum gt_dma_data_direction dir);

gt_dma_buf *gt_dma_buf_get_from_attach(gt_dma_buf_attachment *attach);

void gt_get_dma_buf(gt_dma_buf *dmabuf);

int gt_dma_buf_fd(gt_dma_buf *dmabuf, int flags);

gt_dma_buf *gt_dma_buf_get(int fd);

void gt_dma_buf_put(gt_dma_buf *dmabuf);

uint64_t gt_dma_buf_size(gt_dma_buf *dmabuf);

void *gt_dma_buf_get_priv(gt_dma_buf *dmabuf);

void gt_dma_buf_set_priv(gt_dma_buf *dmabuf, void* priv);

bool gt_dma_buf_is_pmr_ops(gt_dma_buf *dmabuf);

gt_dma_buf *gt_pmr_dma_buf_export(void *priv, uint64_t size, int flags);

void gt_dma_cache_sync(void *dev, void *vaddr, uint64_t size, enum gt_dma_data_direction dir);
#endif
