#ifndef __PDP0_COMMON_H
#define __PDP0_COMMON_H

#include "gt_debug.h"
#include "gt_misc.h"
#include "gt_plat_dev.h"
#include "gt_task.h"
#include "gt_lock.h"
#include "gt_timer.h"
#include "gt_mm.h"
#include "gt_waitqueue.h"
#include "gt_drm.h"
#include "gt_drm_mode.h"
#include "gt_fs.h"
#include "gt_interrupt.h"


struct pdp0_plane_config {
	uint64_t color;
	uint64_t pixel_alpha;
	uint64_t pre_alpha;
	uint64_t layer_alpha;
	uint64_t wh_scaler;
	uint32_t rotation;
};

struct pdp0_layer {
	u8 name[16];
	u16 id;
	u16 base;
	u16 ptr;
	u16 stride_offset;
};

struct pdp0_se_config {
	u8 scale_enable:1;
	u8 enhancer_enable:1;
	u8 hcoeff:3;
	u8 vcoeff:3;
	u8 plane_src_id;
	u16 input_w, input_h;
	u16 output_w, output_h;
	u32 h_init_phase, h_delta_phase;
	u32 v_init_phase, v_delta_phase;
};

struct gtdpu_pdp0_hw_device *pdp0_crtc_to_hwdev(gt_drm_crtc *crtc);
struct pdp0_plane_config *pdp0_plane_state_get_config(gt_drm_plane *plane);
u8 *pdp0_plane_state_get_format(gt_drm_plane *plane);
u8 *pdp0_plane_state_get_nplane(gt_drm_plane *plane);
const struct pdp0_layer *pdp0_plane_get_layer(gt_drm_plane *plane);
struct gtdpu_pdp0_hw_device *pdp0_plane_get_hwdev(gt_drm_plane *plane);
u32 *pdp0_crtc_state_get_gamma_coeffs(gt_drm_crtc_state *crtc_state);
bool pdp0_crtc_state_get_coladj_en(gt_drm_crtc_state *crtc_state);
struct pdp0_se_config *pdp0_crtc_state_get_scaler_config(gt_drm_crtc_state *crtc_state);
struct pdp0_crtc_config *pdp0_crtc_state_get_priv_config(gt_drm_crtc_state *crtc_state);
u32 *pdp0_crtc_state_get_coloradj_coeffs(gt_drm_crtc_state *crtc_state);
u16 pdp0_get_plane_layer_base(struct gtdpu_pdp0_hw_device *hwdev, int num);
void *pdp0_get_gt_framebuffer(void *fb);

unsigned short gt_drm_display_mode_get_vdisplay(
		const gt_drm_display_mode *gt_dmode);

unsigned short gt_drm_display_mode_get_hdisplay(
		const gt_drm_display_mode *gt_dmode);


extern void gt_drm_display_mode_to_videomode(const gt_drm_display_mode *gt_dmode,
															gt_videomode *gt_vm);

u32 pdp0_set_alpha_blend_mode(gt_drm_plane *gt_plane, gt_drm_framebuffer *gt_fb,
									 u32 layer_ctrl);
bool pdp0_is_alpha_support(uint32_t format);
unsigned long pdp0_gem_get_dev_paddr(gt_drm_gem_object *obj);
unsigned long pdp0_gem_get_cpu_paddr(gt_drm_gem_object *obj);
u64 pdp0_get_fb_dev_paddr(struct gtdpu_pdp0_hw_device *hwdev, void *gt_fb);
u64 pdp0_get_fb_cpu_paddr(struct gtdpu_pdp0_hw_device *hwdev, void *gt_fb);


u32 GT_PADDING_ALIGN_SIZE_FUNC(void);
u32 GT_PDP0_CONFIG_VALID_DONE_FUNC(void);
u32 GT_PDP0_FS_BLOCK_FUNC(void);
u32 GT_PDP0_FC_BLOCK_FUNC(void);
u32 GT_PDP0_FD_BLOCK_FUNC(void);



#define GT_PADDING_ALIGN_SIZE           GT_PADDING_ALIGN_SIZE_FUNC()
#define GT_PDP0_CONFIG_VALID_DONE       GT_PDP0_CONFIG_VALID_DONE_FUNC()

#endif

